/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "desktopbutton.h"
#include <KDE/KIcon>
#include <KDE/KLocale>
#include <KDE/KWindowSystem>

#include <QX11Info>

DesktopButton::DesktopButton( QWidget *parent ) : QToolButton(parent)
{
    setObjectName("ShowDesktopButton");
    setCheckable( true );
    setChecked( false );
    setIcon(KIcon("user-desktop"));
    connect( this, SIGNAL(toggled(bool)), this, SLOT(showHideDesktop(bool)) );
}

DesktopButton::~DesktopButton()
{

}

void DesktopButton::showHideDesktop( bool showOrHide )
{
     if( showOrHide )
     {
       m_iconifiedList.clear();
	foreach(WId wid, KWindowSystem::windows())
	{
	    NETWinInfo info(QX11Info::display(), wid, QX11Info::appRootWindow(), NET::XAWMState | NET::WMDesktop);
	    if (info.mappingState() == NET::Visible && (info.desktop() == NETWinInfo::OnAllDesktops  || info.desktop() == (int) KWindowSystem::currentDesktop()) )
	    {
	      m_iconifiedList.append(wid);
	    }
	}

        foreach(WId wid, m_iconifiedList)
	{
	    KWindowSystem::minimizeWindow(wid, false);
	}
    }
    else
    {
        foreach(WId wid, m_iconifiedList)
	{
	  KWindowSystem::unminimizeWindow(wid, false);
	}
    }
}
