/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "launcherbar.h"

#include <KDE/KHistoryComboBox>
#include <KDE/KLocale>
#include <KDE/KRun>
#include <KDE/KMessageBox>
#include <KDE/KSharedConfig>
#include <KDE/KConfigGroup>
#include <KDE/KApplication>
#include <KDE/KUrlPixmapProvider>

#include <kdebug.h>

#include <kurifilter.h>
#include <kdebug.h>
#include <QHBoxLayout>
#include <QLabel>
#include <QEvent>
#include <QLineEdit>
#include <QStringList>

LauncherBar::LauncherBar( QWidget *parent ) : QFrame(parent)
{
    setObjectName("QuickLauncher");
    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->setSpacing(0);
    layout->setContentsMargins(4,0,4,0);

    QLabel *text = new QLabel(i18n("Run:"), this);
    m_input = new KHistoryComboBox(this);
    m_input->setFocus();
    m_input->clearEditText();
    m_input->installEventFilter(this);
    m_input->lineEdit()->installEventFilter(this);
    m_input->setPixmapProvider( new KUrlPixmapProvider );

    connect(m_input, SIGNAL(activated(const QString&)),
            this, SLOT(execute(const QString&)));
    layout->addWidget( text );
    layout->addWidget( m_input,1);

    KConfigGroup grp = KSharedConfig::openConfig("blazer")->group("History");
    QStringList items = grp.readPathEntry("Items", QStringList() );
    QStringList completion = grp.readPathEntry("Completion", QStringList() );

    m_input->completionObject()->setItems(completion);
    m_input->setHistoryItems(items);

}

LauncherBar::~LauncherBar()
{
    KConfigGroup grp = KSharedConfig::openConfig("blazer")->group("History");
    grp.writePathEntry("Completion", m_input->completionObject()->items());
    grp.writePathEntry("Items", m_input->historyItems());

}

void LauncherBar::execute( const QString &execLine )
{
    KUriFilterData execLineData( execLine.trimmed() );
    KUriFilter::self()->filterUri( execLineData, QStringList() << "kurisearchfilter" << "kshorturifilter" );
    QString args;
    QString command = (execLineData.uri().isLocalFile() ? execLineData.uri().path():execLineData.uri().url());

    // Nothing interesting. Quit!
    if ( command.isEmpty() )
    {
        return;
    }
    else
    {
        m_input->addToHistory( m_input->currentText() );
        switch( execLineData.uriType() )
        {
        case KUriFilterData::LocalFile:
        case KUriFilterData::LocalDir:
        case KUriFilterData::NetProtocol:
        case KUriFilterData::Help:
        {
            new KRun( execLineData.uri(), this );
            break;
        }
        case KUriFilterData::Executable:
        case KUriFilterData::Shell:
        {
            args = command;
            if( execLineData.hasArgsAndOptions() )
              command += execLineData.argsAndOptions();
            if (!KRun::runCommand( command, args, "", this ))
            {
                KMessageBox::detailedSorry( this, i18n("Failed to run command"),
                                                  i18n("The command %1 failed to execute.", command ));
                m_input->removeFromHistory( m_input->currentText() );
            }

            break;
        }
        case KUriFilterData::Unknown:
        case KUriFilterData::Error:
        default:
            kDebug() << "Unknown command" << command << args;
            KMessageBox::detailedSorry( this, i18n("Failed to run command"),
                                              i18n("Unable to determine how to run %1.", command) );
            m_input->removeFromHistory( m_input->currentText() );
            break;
        }
    }

    m_input->clearEditText();
    return;
}

bool LauncherBar::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::MouseButtonPress )
    {
        m_input->lineEdit()->activateWindow();
        return true;
    }
    else
    {
        return QObject::eventFilter(obj, event);
    }
}
