/*  This file is part of the KDE Libraries
 *  Copyright (C)  2006   Olivier Goffart <ogoffart @ kde.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef KCOLORSETTINGSITEM
#define KCOLORSETTINGSITEM

#include <QtCore/QFlags>
#include <QtCore/QSharedDataPointer>
#include <QtGui/QColor>
#include <QtGui/QPixmap>
#include <QtGui/QFont>


class QColor;
class QFont;
class QPixmap;
class KConfigGroup;

class KColorSettingsItem
{
	public: 
		enum StyleFlag 
		{
			ForegroundColor=0x02,
			Font = 0x04,
			BackgroundColor=0x10,
//			BackgroundImage=0x20,
//			SelectedBackgroundColor=0x100,
//			SelectedForegroundColor=0x200,
		};
		Q_DECLARE_FLAGS(StyleFlags,StyleFlag)

		KColorSettingsItem();
		KColorSettingsItem(const KColorSettingsItem&);
		~KColorSettingsItem();
		KColorSettingsItem &operator=(const KColorSettingsItem&);
				
		QColor foregroundColor() const;
		QFont font() const ;
		QColor backgroundColor() const ;
/*		QPixmap backgroundImage() const;
		QColor selectedForegroundColor() const;
		QColor selectedBackgroundColor() const;*/
		
		void setForegroundColor(const QColor &);
		void setFont(const QFont &);
		void setBackgroundColor(const QColor &);
/*		void setBackgroundImage(const QPixmap&);
		void setSelectedForegroundColor(const QColor&);
		void setSelectedBackgroundColor(const QColor&);*/
		
		void setParent(const KColorSettingsItem *);
		const KColorSettingsItem* parent() const;
		

		
		StyleFlags mask() const;
		
		/**
		 * the group must be set,   i recommand the usage of KConfigGroup
		 */
		void save(const QString &key ,KConfigGroup *config) const;
		void load(const QString &key ,KConfigGroup *configLocal, KConfigGroup *configDef=0);
		
		
//		QVarient value(RoleMask m);
	private:
		class KColorSettingsItemPrivate;	
		QSharedDataPointer<KColorSettingsItemPrivate> d;
};


 Q_DECLARE_OPERATORS_FOR_FLAGS(KColorSettingsItem::StyleFlags)
		 
#endif
