/*
 * KDE Composite Manager, 
 * Copyright (C) 2003-2005 Fredrik Höglund <fredrik@kde.org>
 *
 * Based on xcompmgr,
 * Copyright (C) 2003 Keith Packard <keithp@keithp.com>
 *
 * Internal structure and naming style inspired by KWin,
 * Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 * Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Fredrik Höglund not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Fredrik Höglund makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * FREDRIK HÖGLUND DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL FREDRIK HÖGLUND BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef __ANIMATION_H
#define __ANIMATION_H

#include <qdatetime.h>
#include "globaldefs.h"

class Client;

class Animation
{
	public:
		Animation( Client *client, AnimationType type, unsigned int flags = NoFlags );
		~Animation();

		void update();

		bool isDone() const { return mDone; }
		Client *client() const { return mClient; }
		AnimationType type() const { return mType; }
		void setType( AnimationType type );
		unsigned int flags() const { return mFlags; }
		void setFlags( unsigned int flags ) { mFlags |= flags; }
		void clearFlags( unsigned int flags ) { mFlags &= ~flags; }

	private:
		void setIsDone( bool v ) { mDone = v; }

		Client *mClient;
		AnimationType mType;
		unsigned int mFlags;
		bool mDone:1;
		QTime mTime;
		double mIncrement;
};

#endif
