/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef KLIPBOARD_MODEL_H
#define KLIPBOARD_MODEL_H

#include <QAbstractItemModel>
#include <Plasma/DataEngine>

class KlipboardModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    enum DataRole
    {
        UuidRole = Qt::UserRole,
        ClipboardTypeRole = Qt::UserRole + 1,
        AlwaysShowClipboardTypeRole = Qt::UserRole + 2
    };

    explicit KlipboardModel(const Plasma::DataEngine* dataEngine, QObject* parent = 0);

    /**
     * Given an uuid returns a model index which represents an item with this uuid.
     * If no such uuid is found, invalid index will be returned
     */
    QModelIndex indexForUuid(const QString& uuid) const;

    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
    int rowCount(const QModelIndex& parent = QModelIndex()) const;
    int columnCount(const QModelIndex& parent = QModelIndex()) const;
    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex& index) const;

private slots:
    void onClipEntryAdded(const QString& sourceName);
    void onClipEntryRemoved(const QString& sourceName);

private:
    const Plasma::DataEngine* m_dataEngine;

    /**
     * Contains mapping of uuid to the row occupied by that entry
     */
    QHash<QString, int> m_uuidRowMap;
};

#endif
