/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef IDENTIFIER_H
#define IDENTIFIER_H

#include <QtCore/QObject>
#include <QtCore/QUrl>

namespace Soprano {
    class Statement;
}

namespace Nepomuk {

    namespace Sync {
        class ChangeLog;
        class SyncFile;
    }
    class IdentificationThread;

    class Identifier : public QObject
    {
        Q_OBJECT
        Q_CLASSINFO("D-Bus Interface", "org.kde.nepomuk.services.nepomukbackupsync.Identifier")
    public :
        Identifier( QObject* parent = 0 );
        virtual ~Identifier();

        void test();
    Q_SIGNALS:
        void identified( int id, const QString & oldUri, const QString & newUri );
        void notIdentified( int id, const QString & serializedStatements );
        void processed( const Nepomuk::Sync::ChangeLog & logFile );

    public Q_SLOTS:
        int process( const Sync::SyncFile & sf );
        bool ignore( int id, const QString & url, bool ignoreSubDirectories );

        /**
         * Add the (oldUri, newUri) pair to the identifier list
         * @param oldUri has to be a nepomuk:/res/
         * @param newUri can be a nepomuk:/res/ or file:/
         */
        bool identify( int id, const QString & oldUri, const QString & newUri );

    private slots:
        void notIdentified( int id, const QList<Soprano::Statement> & st );
        void identified( int id, const QUrl & oldUri, const QUrl & newUri );
    private :
        IdentificationThread * m_thread;
    };

}

#endif // IDENTIFIER_H
