/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "syncmanager.h"
#include "identifier.h"
#include "merger.h"
#include "syncfile.h"
#include "mergerequest.h"
#include "changelog.h"
#include "diffgenerator.h"
#include "nao.h"
#include "syncmanageradaptor.h"
#include "tools.h"

#include <QtDBus/QDBusConnection>

#include <Nepomuk/Resource>
#include <Nepomuk/Variant>
#include "logstorage.h"

Nepomuk::SyncManager::SyncManager(Nepomuk::Identifier* ident, Nepomuk::Merger* merger, QObject* parent)
    : QObject( parent ),
      m_identifier( ident ),
      m_merger( merger )
{
    new SyncManagerAdaptor( this );
    // Register DBus Object
    QDBusConnection dbus = QDBusConnection::sessionBus();
    dbus.registerObject( QLatin1String("/syncmanager"), this );
}


Nepomuk::SyncManager::~SyncManager()
{
}


int Nepomuk::SyncManager::sync(const QString& url)
{
    return sync( QUrl( url ) );
}


int Nepomuk::SyncManager::sync(const QUrl& url)
{
    Sync::SyncFile syncFile( url );
    return m_identifier->process( syncFile );
}


void Nepomuk::SyncManager::createSyncFile( const QString& url, const QString& startTime )
{
    Sync::ChangeLog logFile = LogStorage::instance()->getChangeLog( startTime );

    Sync::SyncFile sf( logFile, ResourceManager::instance()->mainModel() );
    sf.save( url );
}


void Nepomuk::SyncManager::createSyncFile( const QUrl& outputUrl, const QList<QString>& urls)
{
    QSet<QUrl> nepomukUris;
    QDateTime min = QDateTime::currentDateTime();

    foreach( const QString & urlString, urls ) {
        QUrl url( urlString );

        Resource res( url );
        QDateTime dt = res.property( Nepomuk::Vocabulary::NAO::created() ).toDateTime();
        if( dt < min )
            min = dt;

        nepomukUris.insert( res.resourceUri() );
    }

    createSyncFile( outputUrl, nepomukUris, min );
}


void Nepomuk::SyncManager::createSyncFile(const QUrl& outputUrl, QSet<QUrl> & nepomukUris, const QDateTime& min)
{
    Sync::ChangeLog logFile = LogStorage::instance()->getChangeLog( min );

    QList<Sync::ChangeLogRecord> records = logFile.toList();
    foreach( const Sync::ChangeLogRecord & r, records ) {
        const QUrl & objectUri = r.st().object().uri();
        if( nepomukUris.contains( objectUri ) ) {
            if( objectUri.scheme() == QLatin1String("nepomuk") ) {
                nepomukUris.insert( objectUri );
            }
        }
    }

    logFile.filter( nepomukUris );

    Sync::SyncFile sf( logFile, ResourceManager::instance()->mainModel() );
    sf.save( outputUrl );
}


void Nepomuk::SyncManager::createFirstSyncFile(const QUrl& outputUrl) const
{
    Sync::SyncFile sf = Nepomuk::createFirstSyncFile();
    sf.save( outputUrl );
}


#include "syncmanager.moc"
