/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef BACKUPMANAGER_H
#define BACKUPMANAGER_H

#include <QtCore/QObject>

class QDateTime;

namespace Nepomuk {

    class Identifier;
    class Merger;

    class BackupManager : public QObject
    {
        Q_OBJECT
        Q_CLASSINFO("D-Bus Interface", "org.kde.nepomuk.services.nepomukbackupsync.BackupManager")

    public:
        BackupManager(Nepomuk::Identifier* ident, Nepomuk::Merger* merger, QObject* parent = 0);
        virtual ~BackupManager();

    public slots:
        void backup( const QString & url = QString() );
        void restore( const QString & url = QString() );

        QStringList checkpoints( const QString & url = QString() );
        void revertCheckpointsTill( const QString & checkPoint, const QString & url = QString() );
        void revertSingleCheckpoint( const QString & checkPoint, const QString & url = QString() );
        
        void freshBackup( const QString & url = QString() );

    signals:
        void backupDone();
        void backupNotRequired(const QDateTime & dateTime);
        
    private:
        Identifier * m_identifier;
        Merger * m_merger;
    };

}
#endif // BACKUPMANAGER_H
