/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "resourcehash.h"

Nepomuk::ResourceHash Nepomuk::convertToResourceHash(const QList< Soprano::Statement >& list)
{
    //
    // Categorize on the basis on subject uri for easier look up
    //
    QMultiHash<QUrl, Soprano::Statement> stHash;
    foreach( const Soprano::Statement & st, list ) {
        QUrl uri = st.subject().uri();

        stHash.insert( uri, st );
    }

    //
    // Convert them into a better format --> ResourceHash
    //
    ResourceHash resources;

    QHashIterator<QUrl, Soprano::Statement> it( stHash );
    while( it.hasNext() ) {
        it.next();
        QUrl uri = it.key();

        QList<Soprano::Statement> statements = stHash.values( uri );
        foreach( const Soprano::Statement & st, statements ) {
            QUrl pred = st.predicate().uri();
            Soprano::Node obj = st.object();

            // Translate the home url
            if( obj.isResource() && obj.uri().scheme() == QLatin1String("file") )
                obj = Soprano::Node( translateUri( obj.uri() ) );

            rs.propHash.insert( pred, obj );
        }

        resources.insert( rs.uri, rs );
    }

    return resources;
}
