/* This file is part of the KDE Project
   Copyright (c) 2009 Daniel Winter <dw@danielwinter.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _NEPOMUK_FILESYSTEM_MANAGEMENT_H_
#define _NEPOMUK_FILESYSTEM_MANAGEMENT_H_

#include <QList>
#include <QHash>
#include <QtCore/QString>
#include <QtCore/QVariantList>
#include <QUrl>

#include <Nepomuk/Service>

#include "abstractfilesystemhandler.h"


namespace Nepomuk {

    class MountSignalFilter;

    class FilesystemManagementService : public Service
    {
        Q_OBJECT
        Q_CLASSINFO( "D-Bus Interface", "org.kde.nepomuk.FilesystemManagement" )

        public:
            FilesystemManagementService( QObject* parent, const QVariantList& );
            ~FilesystemManagementService();


        private Q_SLOTS:
            void slotFilesystemMounted( Filesystem fs );
            void slotFilesystemUnmounted( Filesystem fs );

        private:
            void initFilesystemList();
            bool addFilesystemToList(const Filesystem& fs );
            void writeAllFilesystemsToNepomuk();
            void resetFilesystemsInNepomuk() const;

        private:
            QList<AbstractFilesystemHandler*> m_handlerList;
            QHash<QString, Filesystem> m_filesystemList;
            MountSignalFilter* m_signalFilter;

    };
}

#endif
