/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "querybuilderwidgetdesignerplugin.h"
#include "querybuilderwidget.h"

#include <QtCore/QtPlugin>
#include <QIcon>

#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/Query/ResourceTerm>
#include <Nepomuk/Types/Class>

#include <Soprano/Vocabulary/NAO>


QueryBuilderWidgetDesignerPlugin::QueryBuilderWidgetDesignerPlugin( QObject* parent )
  : QObject( parent )
{
}

QWidget* QueryBuilderWidgetDesignerPlugin::createWidget( QWidget* parent )
{
    Nepomuk::Query::QueryBuilderWidget* widget = new Nepomuk::Query::QueryBuilderWidget( parent );
    // init with some example data useful in the preview inside Qt Designer
    // this data will be only used there, not in the resulting view in the program.
    Nepomuk::Query::ComparisonTerm coreTerm(Soprano::Vocabulary::NAO::hasTag(),
                                            Nepomuk::Query::ResourceTerm());
    Nepomuk::Query::Query query(coreTerm);
    query.setLimit(50);
    widget->setQuery(query);
    return widget;
}

QString QueryBuilderWidgetDesignerPlugin::group() const
{
    return QString::fromLatin1("Nepomuk (KDE)");
}

QIcon QueryBuilderWidgetDesignerPlugin::icon() const
{
    return QIcon();
}

QString QueryBuilderWidgetDesignerPlugin::includeFile() const
{
    return QString::fromLatin1("nepomuk/querybuilderwidget.h");
}

QString QueryBuilderWidgetDesignerPlugin::toolTip() const
{
    return QString::fromLatin1("A widget to build simple Nepomuk queries");
}

QString QueryBuilderWidgetDesignerPlugin::whatsThis() const
{
    return QString::fromLatin1("A widget for building Nepomuk queries."
                               "It uses ReseourceSearchWidget (with a simple query editor and a facet widget) to configure the query.");
}

bool QueryBuilderWidgetDesignerPlugin::isContainer() const
{
    return false;
}

QString QueryBuilderWidgetDesignerPlugin::name() const
{
    return QString::fromLatin1("Nepomuk::QueryBuilderWidget");
}

// export macro, takes the name of the plugin module and the class name
Q_EXPORT_PLUGIN2( querybuilderwidgetplugin, QueryBuilderWidgetDesignerPlugin )

#include "querybuilderwidgetdesignerplugin.moc"
