/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourceitemwidget.h"

#include "resourceitemview.h"
#include "resourceitemdelegate.h"
#include "resourcelistmodel.h"

#include <QStringListModel>
#include <QModelIndex>
#include <QModelIndexList>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QList>
#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include <QDockWidget>
#include <QSizePolicy>
#include <QShowEvent>
#include <QSortFilterProxyModel>

#include <KLineEdit>
#include <KDebug>
#include <KPushButton>

#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/QueryParser>
#include <Nepomuk/Query/QueryServiceClient>
#include <Nepomuk/Query/Result>
#include <Nepomuk/Query/Term>
#include <Nepomuk/Query/AndTerm>
#include <Nepomuk/Resource>
#include <Nepomuk/Types/Class>


class Nepomuk::ResourceItemWidget::Private
{
public:
    Nepomuk::Query::Query query;

    Nepomuk::ResourceListModel *model;
    QSortFilterProxyModel *proxyModel;
    QList< Nepomuk::Query::Result> resources;

    Nepomuk::Query::QueryServiceClient *queryClient;
};

Nepomuk::ResourceItemWidget::ResourceItemWidget(QWidget *parent)
    : ResourceItemView(parent),
      d(new Private())
{
    //query client
    d->queryClient = new Nepomuk::Query::QueryServiceClient(this);
    connect(d->queryClient, SIGNAL(newEntries(QList<Nepomuk::Query::Result>)),
            this, SLOT(newEntries(QList<Nepomuk::Query::Result>)));
    connect(d->queryClient, SIGNAL(finishedListing()),
            this, SLOT(finishedListing()));

    //setup MVC
    Nepomuk::ResourceItemDelegate *delegate = new Nepomuk::ResourceItemDelegate(this);
    setItemDelegate(delegate);
    d->model = new Nepomuk::ResourceListModel(this);
    /*d->proxyModel = new QSortFilterProxyModel(this);
    d->proxyModel->setSourceModel(d->model);
    //d->proxyModel->setDynamicSortFilter(true);//this doesn't seem to work
    d->proxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    */
    setModel(d->model);
}

Nepomuk::ResourceItemWidget::~ResourceItemWidget()
{
    delete d;
}

void Nepomuk::ResourceItemWidget::setQuery(const Nepomuk::Query::Query &query)
{
    d->query = query;
    runQuery();
}

Nepomuk::Resource Nepomuk::ResourceItemWidget::currentResource() const
{
    //BUG: the next item is returned always
    //int index = d->proxyModel->mapToSource(currentIndex()).row();
    int index = currentIndex().row();
    if(index >= 0 && index < d->resources.count()) {
        return d->resources.at(index).resource();
    }
    else {
        return Nepomuk::Resource();
    }
}

QList<Nepomuk::Resource> Nepomuk::ResourceItemWidget::selectedResources()
{
    QList<Nepomuk::Resource> resourceList;

    foreach(QModelIndex index, selectionModel()->selectedIndexes()) {
        //QModelIndex sourceIndex = d->proxyModel->mapToSource(index);
        QModelIndex sourceIndex = index;
        resourceList.append(d->resources.at(sourceIndex.row()).resource());
    }

    return resourceList;
}

void Nepomuk::ResourceItemWidget::runQuery()
{
    d->resources.clear();
    d->model->setResources( QList<Query::Result>() );
    if(d->query.isValid()) {
        kDebug() << "********** Running query: " << d->query << d->query.toSparqlQuery();
        d->queryClient->query(d->query);
    }
    else {
        kDebug() << "********** NOT Running invalid query:" << d->query << d->query.toSparqlQuery();;
    }
}

void Nepomuk::ResourceItemWidget::newEntries(const QList<Nepomuk::Query::Result> &entries)
{
    foreach(Nepomuk::Query::Result result, entries) {
        if (!d->resources.contains(result)) {
            d->resources.append(result);
        }
    }
    //FIME: this causes flickering. It will be needed though, for proper asynch querying.
    //d->model->setResources(d->resources);
}

void Nepomuk::ResourceItemWidget::finishedListing()
{
    d->queryClient->close();
    d->model->setResources(d->resources);
    //d->proxyModel->sort(0);
    kDebug() << "*** ResourceListModel rowcount:" << d->model->rowCount();
}
