/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RESOURCEITEMWIDGET_H
#define RESOURCEITEMWIDGET_H

#include "resourceitemview.h"

namespace Nepomuk {

    namespace Query
    {
        class Query;
        class Result;
    }

    class Resource;

    /**
      * Provides a list widget based on Resource items (handling a ResourceItemModel internally).
      * The list is populated automatically based on a given query.
      */
    class ResourceItemWidget : public ResourceItemView
    {
        Q_OBJECT

    public:
        ResourceItemWidget(QWidget *parent = 0);
        ~ResourceItemWidget();
        /**
          * Sets the query on the item widget to @p term
          * @see query()
          */
        void setQuery(const Nepomuk::Query::Query &query);

        /**
          * @returns the current query.
          * @see setQuery()
          */
        Nepomuk::Query::Query query() const;

        /**
          * @returns the current resource if any, an invalid resource if none is selected
          */
        Nepomuk::Resource currentResource() const;

        /**
          * @returns a list of all selected and non-hidden items in the view
          */
        QList<Nepomuk::Resource> selectedResources();

    private:
        class Private;
        Private * const d;

    private Q_SLOTS:
        /**
          * Clears the view and runs the query.
          */
        void runQuery();

        /**
          * Adds new entries to the entry list and updates the model
          * @param entries the list of the entries
          */
        void newEntries(const QList< Nepomuk::Query::Result > &entries);

        /**
          * Closes the query client
          */
        void finishedListing();
    };

}

#endif // RESOURCEITEMWIDGET_H
