/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "misctests.h"
#include "../src/abstractresourceeditplugin.h"
#include "../src/genericresourceeditplugin.h"
#include "../src/resourceeditwidget.h"
#include "../src/resourcelistmodel.h"
#include "../src/resourceitemview.h"
#include "../src/facet.h"
#include "../src/facetmodel.h"
#include "../src/facetwidget.h"
//#include "../src/querybuilderwidget.h"

#include "modeltest.h"

#include <Nepomuk/Resource>
#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/GroupTerm>
#include <Nepomuk/Query/Term>
#include <Nepomuk/Query/AndTerm>
#include <Nepomuk/Query/OrTerm>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/Query/LiteralTerm>
#include <Nepomuk/Query/ResourceTerm>
#include <Nepomuk/Query/ResourceTypeTerm>
#include <Nepomuk/Tag>
#include <Nepomuk/Types/Class>

#include <Soprano/LiteralValue>

#include <QUrl>

#include <KDebug>


void MiscTests::testAbstractResourceEditPlugin()
{
    Nepomuk::AbstractResourceEditPlugin *arep = new Nepomuk::GenericResourceEditPlugin();

    Nepomuk::Resource resource;
    resource.setLabel("test");
    arep->setResource(resource);
#warning enable/disable editing, save, test
    //arep->changeEditingMode(Nepomuk::AbstractResourceEditPlugin::StartEditing);
    //arep->changeEditingMode(Nepomuk::AbstractResourceEditPlugin::SaveChanges);

    QCOMPARE(resource, arep->resource());

    delete arep;
    resource.remove();
}

void MiscTests::testResourceEditWidget()
{
    Nepomuk::ResourceEditWidget *rew = new Nepomuk::ResourceEditWidget();

    Nepomuk::Resource resource;
    resource.setLabel("test");
    rew->setResource(resource);

    QCOMPARE(resource, rew->resource());

    delete rew;
    resource.remove();
}

void MiscTests::testResourceListModel()
{
#ifdef MODELTEST_H
    Nepomuk::ResourceListModel *model = new Nepomuk::ResourceListModel();
    new ModelTest(model, this);
    delete model;
#endif
}

void MiscTests::testResourceItemViewSelections()
{
}

void MiscTests::testFacetModel()
{
#ifdef MODELTEST_H
    Nepomuk::Utils::FacetModel *model = new Nepomuk::Utils::FacetModel();
//    model->setFacets(model->createDefaultFacets());//populateFromQuery(Nepomuk::Query::Query());
    new ModelTest(model, this);
#endif
}

void MiscTests::testFacetsWidget()
{
    Nepomuk::Utils::FacetWidget *widget = new Nepomuk::Utils::FacetWidget();
    Nepomuk::Query::Query query;
    Nepomuk::Query::AndTerm andTerm;
    Nepomuk::Query::AndTerm tagTerm;

#warning aoszkar: This is just a local test on my machine
    tagTerm.addSubTerm(Nepomuk::Query::ComparisonTerm(Nepomuk::Types::Property(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#hasTag")),
                                                           Nepomuk::Query::ResourceTerm(QUrl("nepomuk:/res/7e1e98ce-6b7f-4d4b-8cbd-1e312596e8ee"))));
    tagTerm.addSubTerm(Nepomuk::Query::ComparisonTerm(Nepomuk::Types::Property(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#hasTag")),
                                                           Nepomuk::Query::ResourceTerm(QUrl("nepomuk:/res/4b702846-62ef-4bc0-b6e5-049f3211b3b7"))));

    andTerm.addSubTerm(Nepomuk::Query::ResourceTypeTerm(Nepomuk::Types::Class(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact"))));
    andTerm.addSubTerm(tagTerm);
    query.setTerm(andTerm);
    query.setLimit(100);
    query = query.optimized();
    widget->extractFacetsFromTerm(query.term());

    //Test 1 -------
    //check if we set the query constructed from the Facet widget, we get the same constructed query again
    Nepomuk::Query::Query before(widget->queryTerm());
    widget->extractFacetsFromTerm(widget->queryTerm());
    Nepomuk::Query::Query after(widget->queryTerm());
    QCOMPARE(before, after);

    delete widget;
    widget = new Nepomuk::Utils::FacetWidget();

    //Test 2 -------
    //Check if the initial query is built by the facets and the baseQuery
    widget->extractFacetsFromTerm(query.term());
    //Nepomuk::Query::Term query((query.optimized()));
    Nepomuk::Query::Query facetQuery(widget->queryTerm());
    kDebug() << "*** --- *** initial query: " << query.optimized();
    kDebug() << "*** --- *** facets: " << facetQuery.optimized();
    //WARNING: The test fails because equality check does not actually check whether the queries are equivalent
    QCOMPARE(query.limit(), facetQuery.limit());
    QCOMPARE(query.optimized(), facetQuery.optimized());
}

#if 0
//Tests ResourceSearchWidget as well
void MiscTests::testQueryBuilderWidget()
{
    Nepomuk::Query::QueryBuilderWidget *widget = new Nepomuk::Query::QueryBuilderWidget();

    Nepomuk::Query::Query query;
    Nepomuk::Query::AndTerm andTerm;

    Nepomuk::Query::AndTerm tagTerm;

#warning aoszkar: This is just a local test on my machine
    tagTerm.addSubTerm(Nepomuk::Query::ComparisonTerm(Nepomuk::Types::Property(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#hasTag")),
                                                           Nepomuk::Query::ResourceTerm(QUrl("nepomuk:/res/7e1e98ce-6b7f-4d4b-8cbd-1e312596e8ee"))));
    tagTerm.addSubTerm(Nepomuk::Query::ComparisonTerm(Nepomuk::Types::Property(QUrl("http://www.semanticdesktop.org/ontologies/2007/08/15/nao#hasTag")),
                                                           Nepomuk::Query::ResourceTerm(QUrl("nepomuk:/res/4b702846-62ef-4bc0-b6e5-049f3211b3b7"))));

    //this should go to the baseQuery()
    andTerm.addSubTerm(Nepomuk::Query::ResourceTypeTerm(Nepomuk::Types::Class(QUrl("http://www.semanticdesktop.org/ontologies/2007/03/22/nco#Contact"))));

    //this should be picked up by the facets widget
    andTerm.addSubTerm(tagTerm);

    //this should go to the lineedit
    andTerm.addSubTerm(Nepomuk::Query::LiteralTerm(Soprano::LiteralValue("latex")));

    query.setTerm(andTerm);
    widget->setQuery(query);
    //widget->setFacetsWidgetQuery(query);

    kDebug() << "*** query: " << query;
    kDebug() << "*** widget->currentQuery(): " << widget->currentQuery();
    //WARNING: The test fails because query equality check does not actually check whether the queries are equivalent (relies on term equality)
    QCOMPARE(query, widget->currentQuery());
}
#endif
QTEST_KDEMAIN(MiscTests, GUI)
#include "misctests.moc"
