#include <QComboBox>
#include <QHBoxLayout>
#include <QList>
#include <QPushButton>

#include <KIcon>

#include "comparator.h"
#include "searchcriterion.h"
#include "valuewidget.h"

namespace
{
    ValueWidget* dateFactory(QWidget* parent = NULL)
    {
        return new DateWidget(parent);
    }

    ValueWidget* sizeFactory(QWidget* parent = NULL)
    {
        return new SizeWidget(parent);
    }
}

QList<Field> SearchCriterion::fields;

SearchCriterion::SearchCriterion(QWidget* parent) :
    QWidget(parent), layout(new QHBoxLayout(this)), field(new QComboBox(this)), comp(0), value(0), remove(new QPushButton(this))
{
    if(fields.isEmpty())
        buildFieldList();

    field->addItem( i18n("Select...") );

    QList<Field>::const_iterator it;
    for(it = fields.constBegin(); it != fields.constEnd(); ++it)
    {
        field->addItem( it->name() );
    }

    connect( field, SIGNAL(currentIndexChanged(QString)), this, SLOT(fieldChanged(QString)) );
    connect( field, SIGNAL(currentIndexChanged(QString)), this, SLOT(updateQuery()) );
    layout->addWidget(field);

    remove->setIcon( KIcon( "list-remove" ) );
    remove->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    connect( remove, SIGNAL(clicked()), this, SIGNAL(removeCriterion()));
    layout->addWidget(remove);

    setLayout(layout);
}

void SearchCriterion::updateQuery()
{
    const comparator& c = getCurrentComparator();
    if(c.name.compare(invalid.name) == 0) return;
    if(!value) return;

    query = c.prefix + getCurrentField().field() + c.str + value->getValue();

    emit criterionChanged(query);
}

QString SearchCriterion::getSearchText() const
{
    return query;
}

const Field& SearchCriterion::getCurrentField()
{
    return fields.at(field->currentIndex() - 1);
}

const comparator& SearchCriterion::getCurrentComparator()
{
    const Field& field = getCurrentField();

    QList<comparator>::const_iterator it;
    for(it = field.comparators().constBegin(); it != field.comparators().constEnd(); ++it)
    {
        if(it->name == comp->currentText())
            return *it;
    }
    //make gcc happy;
    return invalid;
}

void SearchCriterion::fieldChanged(const QString& fieldStr)
{
    if(!comp)
    {
        comp = new QComboBox(this);
        layout->insertWidget(layout->count() - 1, comp);
    }
    comp->clear();

    comp->addItem( QString( i18n("Select...")));
    QList<comparator>::const_iterator it;
    for(it = getCurrentField().comparators().constBegin(); it != getCurrentField().comparators().constEnd(); ++it)
    {
        comp->addItem(it->name);
    }

    connect( comp, SIGNAL(currentIndexChanged(QString)), this, SLOT(updateQuery()) );

    if(value)
    {
        layout->removeWidget(value->getWidget());
        delete value;
    }
    value = getCurrentField().getWidget(this);
    connect(value, SIGNAL(valueUpdated(QString)), this, SLOT(updateQuery()));

    layout->insertWidget(layout->count() - 1, value->getWidget());

}

void SearchCriterion::buildFieldList()
{
    QList<comparator> c;

    c.append(greater);
    c.append(greateroreq);
    c.append(less);
    c.append(lessoreq);

    Field date( i18n("Date Modified"), "sourceModified", c, &dateFactory);
    Field size( i18n("File size"), "contentSize", c, &sizeFactory);

    fields.append(date);
    fields.append(size);
}
