#include <QPushButton>
#include <QVBoxLayout>
#include <QComboBox>
#include <QSpacerItem>

#include <KIcon>
#include <KDebug>

#include "searchcriterion.h"
#include "searchpane.h"

SearchPane::SearchPane(QWidget *parent) :
        QWidget(parent), layout(new QVBoxLayout)
{
    QPushButton* newCriteria = new QPushButton;
    newCriteria->setIcon( KIcon( "list-add" ) );
    newCriteria->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    connect(newCriteria, SIGNAL(pressed()), this, SLOT(addCriterion()) );

    layout->addWidget(newCriteria, Qt::AlignCenter);

    setLayout(layout);
}

SearchPane::~SearchPane()
{
}

void SearchPane::updateQuery()
{
    QStringList items;
    for(int i = 0; i < layout->count(); ++i)
    {
        QLayoutItem* item = layout->itemAt( i );
        SearchCriterion* sc = dynamic_cast<SearchCriterion*>( item->widget() );
        if(sc)
            items.append( sc->getSearchText() );
    }
    query = items.join(" ");

    kDebug() << "query set to" << query;

    emit queryChanged(query);
}

void SearchPane::addCriterion()
{
    SearchCriterion* criterion = new SearchCriterion;
    connect(criterion, SIGNAL(criterionChanged(QString)), this, SLOT(updateQuery()));
    connect(criterion, SIGNAL(removeCriterion()), this, SLOT(removeCriterion()) );

    layout->insertWidget(layout->count() - 1, criterion);
}

void SearchPane::removeCriterion()
{
    kDebug() << "removing widget" << sender();
    layout->removeWidget(dynamic_cast<QWidget*>(sender()));
    delete sender();
}
