/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "httpget.h"

#include <iostream>

#include <QtCore/QCoreApplication>
#include <QtCore/QDebug>
#include <QtCore/QEventLoop>
#include <QtCore/QTextStream>
#include <QtCore/QUrl>
#include <QtNetwork/QHttp>
#include <QtNetwork/QHttpRequestHeader>
#include <QtNetwork/QHttpResponseHeader>

HttpGet::HttpGet( QObject* parent )
    : QObject( parent )
{
    qDebug() << "HttpGet::HttpGet()";
    loop = new QEventLoop( this );
    http = new QHttp( this );
    connect( http, SIGNAL( requestFinished( int, bool ) ), this, SLOT( requestFinished( int, bool ) ) );
    connect( http, SIGNAL( done( bool ) ), this, SLOT( done( bool ) ) );
}

void HttpGet::get( const QUrl& url )
{
    qDebug() << "HttpGet::execute()";
    QHttpRequestHeader header( "GET", url.path() );
    header.setValue( "Host", url.host() );
    header.setValue( "Accept", "application/rdf+xml" );

    http->setHost( url.host() );
    httpGetId = http->request( header );
}

void HttpGet::requestFinished( int id, bool error )
{
    qDebug() << "HttpGet::requestFinished()";
    if ( id != httpGetId || error ) {
        return;
    }

    QHttp* http = qobject_cast< QHttp* >( sender() );
    const QHttpResponseHeader response = http->lastResponse();

    switch ( response.statusCode() ) {
    case 200:  // OK
    {
        qDebug() << "OK";
        QTextStream output( stdout );
        output << http->readAll();
        break;
    }

    case 301:  // Moved Permanently
    case 302:  // Found
    case 303:  // See Other
    case 307:  // Temporary Redirect
        qDebug() << "Redirect";
        get( QUrl( response.value( "Location" ) ) );
        break;

    default:
        qDebug() << "Unknown";
    }
}

void HttpGet::done( bool error )
{
    qDebug() << "HttpGet::done()";
    loop->quit();
}

void HttpGet::wait()
{
    loop->exec();
}

#include "httpget.moc"
