/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "graphretrievertest.h"

#include "autoptr.h"
#include "graphretriever.h"
#include "testdata.h"

#include <QtTest/QtTest>

#include <Soprano/Model>

using namespace SemClip;

void GraphRetrieverTest::initTestCase()
{
    m_foaf = QUrl( "http://xmlns.com/foaf/0.1/" );
    m_wot = QUrl( "http://xmlns.com/wot/0.1/" );
    m_dc = QUrl( "http://purl.org/dc/elements/1.1/" );
    m_vcard = QUrl( "http://www.w3.org/2006/vcard/ns#" );
}

void GraphRetrieverTest::cleanupTestCase()
{
}

void GraphRetrieverTest::testGet()
{
    // test acceptance of URLs of the same authority
    GraphRetriever same;
    QVERIFY( same.get( m_wot ) );
    QVERIFY( same.get( m_foaf ) );

    // test acceptance of URLs of different authorities
    GraphRetriever different;
    QVERIFY( different.get( m_wot ) );
    QVERIFY( different.get( m_dc ) );
}

void GraphRetrieverTest::testResultModel()
{
    // test with invalid URLs
    GraphRetriever invalid;
    foreach ( QUrl uri, TestData::exampleOntologies() ) {
        invalid.get( uri );
    }
    AutoPtr< Soprano::Model > empty( invalid.model() );
    QVERIFY( !empty.isNull() );
    QVERIFY( empty->isEmpty() );

    // test with valid URLs
    GraphRetriever valid;
    valid.get( m_foaf );
    valid.get( m_vcard );
    AutoPtr< Soprano::Model > model( valid.model() );
    QVERIFY( !model.isNull() );
    QVERIFY( !model->isEmpty() );

    // test with timeout
    GraphRetriever timeout;
    timeout.setTimeoutThreshold( 5 );
    timeout.get( QUrl( "http://www.example.org:800/" ) );
    AutoPtr< Soprano::Model > aborted( timeout.model() );
    QVERIFY( !aborted.isNull() );
    QVERIFY( timeout.hasTimedOut() );
}

void GraphRetrieverTest::testResultSerialization()
{
    // test for GraphUtility::serializeModelToRdfXml covers this
}

QTEST_MAIN( GraphRetrieverTest )

#include "graphretrievertest.moc"
