/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "sparqlmappingstoragetest.h"

#include "mappingelement.h"
#include "sparqlmappingstorage.h"

#include <QtCore/QUrl>
#include <QtTest/QtTest>

#include <qtest_kde.h>

using namespace SemClip;

void SparqlMappingStorageTest::testFindMapping()
{
    const QUrl foaf( "http://xmlns.com/foaf/0.1/" );
    const QUrl vcard( "http://www.w3.org/2006/vcard/ns#" );
    const QUrl invalid( "http://example.org/invalid/" );
    SparqlMappingStorage remote;

    // test finding a non existing mapping
    QVERIFY( !remote.findMapping( foaf, invalid ) );

    // test finding an existing mapping
    const MappingElement* mapping = remote.findMapping( foaf, vcard );
    QVERIFY( mapping );
    QCOMPARE( mapping->sourceUri(), foaf );
    QCOMPARE( mapping->targetUri(), vcard );
    QVERIFY( !mapping->fileName().isEmpty() );
    QVERIFY( mapping->content() );

    // test accessing an unreachable remote mapping storage
    SparqlMappingStorage unreachable( QUrl( "http://www.example.org/whatever" ) );
    QVERIFY( !unreachable.findMapping( foaf, vcard ) );
}

QTEST_KDEMAIN( SparqlMappingStorageTest, NoGUI )

#include "sparqlmappingstoragetest.moc"
