/*
 * This file is part of Search Project.
 *
 * Copyright (C) 2007 Daniele Galdi <daniele.galdi@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef SEARCH_MAIN_WINDOW_H
#define SEARCH_MAIN_WINDOW_H

#include <QLabel>
#include <QLineEdit>
#include <QWidget>
#include <QAbstractItemView>
#include <QAbstractItemModel>

#include <strigi/qtdbus/strigiasyncclient.h>

#include "SearchBar.h"
#include "FilterBar.h"
#include "StrigiHitsModel.h"
#include "TreeView.h"

class SearchMainWindow : public QWidget
{
  Q_OBJECT

public:
  SearchMainWindow( QWidget *parent = 0 );
  ~SearchMainWindow();

private:
  StrigiHitsModel* m_model;
  TreeView* m_tree; 

  StrigiAsyncClient m_client;
  QString m_queryHits;
  QString m_queryCount;

  SearchBar* m_searchBar;
  FilterBar* m_filter;
  
public slots:
  void handleHits( const QString &query, int offset, const QList<StrigiHit> &hits );
  void handleCount( const QString &query, int count );
  void sortBy( const QString& sortBy );

private slots:
  void returnPressed();
};

#endif // SEARCH_MAIN_WINDOW_H
