/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef ABSTRACT_CELL_H_
#define ABSTRACT_CELL_H_

#include <QtGui/QGraphicsWidget>
#include <QVariant>

/**
 * An item to display contact's properties (like its name, photo) and/or provide
 * communication actions (button to start chat, input field for instant message etc).
 */
class AbstractCell: public QGraphicsWidget
{

public:
	/** Constructor, see QGraphicsWidget */
	explicit AbstractCell(QGraphicsItem * parent = 0, Qt::WindowFlags wFlags = 0);

	/** Destructor */
	virtual ~AbstractCell();

	/**
	 * Any of the contact's properties has changed. Must be implemented in
	 * subclasses and should be used to update the cell, if needed, possibly
	 * animated.
	 */
	virtual void propertiesChanged(const QVariantMap &properties) = 0;
};

#endif // ABSTRACT_CELL_H_
