/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "eventcell.h"

#include "contactactions.h"

#include <QtGui/QGraphicsLinearLayout>
#include <QtGui/QAction>
#include <QtCore/QDateTime>

#include "kdebug.h"
#include "kicon.h"

EventCell::EventCell(const QVariantMap &properties, QGraphicsWidget *parent) :
    AbstractCell(parent)
{
    m_iconMessages = new Plasma::IconWidget();
    m_iconMessages->setPreferredSize(QSizeF(22, 22));
    connect(m_iconMessages, SIGNAL(activated()), this, SLOT(readMessages()));
    m_iconMessages->setIcon(KIcon("mail-unread-new"));

    m_iconBirthday = new Plasma::IconWidget();
    m_iconBirthday->setPreferredSize(QSizeF(22, 22));
    m_iconBirthday->setIcon(KIcon("view-pim-calendar"));

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Horizontal, this);
    setLayout(layout);
    layout->addItem(m_iconMessages);
    layout->addItem(m_iconBirthday);

    propertiesChanged(properties);
}

EventCell::~EventCell()
{
    m_iconMessages->deleteLater();
    m_iconBirthday->deleteLater();
}

void EventCell::propertiesChanged(const QVariantMap &properties)
{
    m_id = properties["instant_messaging_id"].toString();

    QStringList pendingMessages = properties["instant_messaging_unread_messages"].toStringList();
    if (pendingMessages.size() > 0)
    {
        m_iconMessages->show();
        QString toolTipText = i18np("<b>One unread message:</b><br />","<b>%1 unread messages:</b><br />",pendingMessages.size());
        foreach(const QString &message, pendingMessages)
        {
            toolTipText += message;
        }

        m_iconMessages->setToolTip(toolTipText);
    }
    else
    {
        m_iconMessages->hide();
        m_iconMessages->setToolTip("");
    }

    QDateTime birthday = properties["birthday"].toDateTime();
    if (birthday.isValid())
    {
        QDate today = QDateTime::currentDateTime().date();
        int difference = birthday.date().dayOfYear() - today.dayOfYear();
        if (difference == 0)
        {
            m_iconBirthday->show();
            m_iconBirthday->setEnabled(true);
            m_iconBirthday->setToolTip(i18n("Today is %1 birthday",properties["name"].toString()));
        }
        else if (difference >= 0 && difference <= 3)
        {
            m_iconBirthday->show();
            m_iconBirthday->setEnabled(false);
            m_iconBirthday->setToolTip(i18np("Only one day left to %2 birthday","Only %1 days left to %2 birthday",difference,properties["name"].toString()));
        }
        else
        {
            m_iconBirthday->hide();
            m_iconBirthday->setEnabled(false);
            m_iconBirthday->setToolTip("");
        }
    }
    else
    {
        m_iconBirthday->hide();
        m_iconBirthday->setEnabled(false);
        m_iconBirthday->setToolTip("");
    }
}

void EventCell::readMessages()
{
    if (!m_id.isEmpty())
    {
        ContactActions::openChatKopete(m_id);
    }
}
