/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "lineeditcell.h"

#include <QtGui/QGraphicsLinearLayout>

#include "kdebug.h"
#include "klocale.h"

LineEditCell::LineEditCell(const QString receiverId, const QString reachableId,
		const QVariantMap &properties, QGraphicsWidget *parent) :
	AbstractCell(parent), m_receiverId(receiverId), m_reachableId(reachableId)
{
	m_lineEdit = new Plasma::LineEdit();
	m_lineEdit->setMinimumWidth(150);
	m_lineEdit->setMaximumHeight(28); // TODO FIXME: use a sane approach for the height

	m_layout = new QGraphicsLinearLayout(Qt::Horizontal);
	setLayout(m_layout);
	m_layout->addItem(m_lineEdit);

	// Eventually emit inputFinished if return pressed and message is non-empty
	connect(m_lineEdit, SIGNAL(returnPressed()), this, SLOT(inputFinished()));

	propertiesChanged(properties);
}

LineEditCell::~LineEditCell()
{
	m_lineEdit->deleteLater();
}

void LineEditCell::propertiesChanged(const QVariantMap &properties)
{
	bool canSendMessage(true);
	if (properties.contains(m_reachableId))
	{
		canSendMessage = properties[m_reachableId].toBool();
	}

	m_lineEdit->setEnabled(canSendMessage);
	m_lineEdit->setToolTip(canSendMessage ? i18n(
			"Type a message and press return to send it") : i18n(
			"This contact cannot be reached at the moment."));

	m_receiver = properties[m_receiverId].toString();
}

void LineEditCell::inputFinished()
{
	if (!m_lineEdit->text().isEmpty() && !m_receiver.isEmpty())
	{
		emit inputFinished(m_receiver, m_lineEdit->text());
		m_lineEdit->setText(QString());
	}
}
