/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "statuscell.h"

#include <QtGui/QGraphicsLinearLayout>
#include <QtGui/QAction>
#include <QtCore/QDateTime>

#include "kicon.h"
#include <Plasma/Animator>
#include <klocale.h>

StatusCell::StatusCell(const QVariantMap &properties, QGraphicsWidget *parent) :
    AbstractCell(parent)
{
    m_iconOnlineStatus = new Plasma::IconWidget();
    m_iconOnlineStatus->setPreferredSize(QSizeF(22, 22));

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Horizontal, this);
    setLayout(layout);
    layout->addItem(m_iconOnlineStatus);

    propertiesChanged(properties);
}

StatusCell::~StatusCell()
{
    m_iconOnlineStatus->deleteLater();
}

void StatusCell::propertiesChanged(const QVariantMap &properties)
{
    QString status = properties["instant_messaging_status"].toString();

    if (status.isEmpty())
    {
        m_iconOnlineStatus->hide();
    }
    else
    {
        m_iconOnlineStatus->show();
        m_iconOnlineStatus->setText(status);

        if (status.contains("online", Qt::CaseInsensitive))
        {
            m_iconOnlineStatus->setIcon(KIcon("user-online"));
        }
        else if (status.contains("offline", Qt::CaseInsensitive))
        {
            m_iconOnlineStatus->setIcon(KIcon("user-offline"));
        }
        else if (status.contains("away", Qt::CaseInsensitive))
        {
            m_iconOnlineStatus->setIcon(KIcon("user-away"));
        }
        else if (status.contains("busy", Qt::CaseInsensitive))
        {
            m_iconOnlineStatus->setIcon(KIcon("user-busy"));
        }
        else if (status.contains("invisible", Qt::CaseInsensitive))
        {
            m_iconOnlineStatus->setIcon(KIcon("user-invisible"));
        }
        else
        {
            m_iconOnlineStatus->setIcon(KIcon("user-offline"));
        }
    }

    QString statusMessage = properties["instant_messaging_status_message"].toString();
    if (statusMessage.isEmpty())
    {
        m_iconOnlineStatus->setToolTip(QString());
    }
    else
    {
        m_iconOnlineStatus->setToolTip(i18n("<b>Status Message:</b><br />%1",statusMessage));
    }

    if (m_oldStatus != status)
    {
        m_oldStatus = status;
        Plasma::Animator::self()->animateItem(m_iconOnlineStatus, Plasma::Animator::AppearAnimation);
    }
}
