/***************************************************************************
 *   Copyright (C) 2009 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef TASKS_GROUPING_MODEL_HEADER
#define TASKS_GROUPING_MODEL_HEADER

#include <Plasma/DataEngine>
#include <QAbstractItemModel>
#include <QModelIndex>
#include <QList>
#include <QHash>

class TasksGroupingModel : public QAbstractItemModel
{

    Q_OBJECT

typedef QList<QList<QString> > hierarchy_type;

public:
    TasksGroupingModel(Plasma::DataEngine *tasksEngine,
                       const QString &propertyName,
                       QObject *parent = 0);

    virtual ~TasksGroupingModel();

    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    int rowCount(const QModelIndex &index = QModelIndex()) const;

    int columnCount(const QModelIndex &index = QModelIndex()) const;

    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;

    QModelIndex parent(const QModelIndex &parent) const;

    bool hasChildren(const QModelIndex &parent = QModelIndex()) const;

    void setGroupingProperty(const QString &propertyName);

protected:

    QVariant categoryData(int role, int categoryIndex) const;
    QVariant taskData(const QString &taskName, int role) const;

private slots:

    void taskAdded(const QString &taskName);
    void taskRemoved(const QString &taskName);
    void dataUpdated(const QString &taskName, const Plasma::DataEngine::Data &taskData);

private:

    QModelIndex indexFromTaskName(const QString &taskName) const;

    void updateBackgroundPrefix(const QString &taskName, const Plasma::DataEngine::Data &taskData);
    void newSourceItem(const QString &taskName,
                       const Plasma::DataEngine::Data &taskData);
    void createNewCategory(const QVariant &theCategory,
                           const QString &taskName,
                           const Plasma::DataEngine::Data &taskData);

    void resetEverything();

    Plasma::DataEngine *m_tasksEngine;
    hierarchy_type m_hierarchy;
    QHash<QString, Plasma::DataEngine::Data> m_tasksData;
    QString m_propertyName;
};

#endif
