/*
 *   Copyright (C) 2007 Matias Costa <m.costacano@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef SMARTGROUPS_H
#define SMARTGROUPS_H

#include <QObject>
#include <QList>
#include <QPointF>

#include "group.h"
#include <plasma/applet.h>

/**
    @author M. Costa <m.costacano@gmail.com>
*/

class StandardDesktop;

class SmartGroups: public QObject
{
    Q_OBJECT
public:
    SmartGroups(StandardDesktop *desktop);

    ~SmartGroups();

    void place(const QList<Plasma::Applet*>& items);

    void learn();

    // Groups
    int groupCount() const;
    int groupIndex(const Plasma::Applet *unit) const;
    const Group& group(const Plasma::Applet *unit) const;
    

    const Group& operator[](int index)
    {
        return m_groups[index];
    };
    
    void clear();
    void remove(const Plasma::Applet *e);
    void reset();

private:
    double euclideanDistance(const Plasma::Applet *a, const Plasma::Applet *b);
    
    double tagsDistance(const Group &c,
                        const QString &tags);
    double euclideanDistance(const QPointF &a, const QPointF &b);
    double groupDistance(const Group &c,
                         const Plasma::Applet *e, double Wt);
    
    int bestGroup(const Plasma::Applet *e);
    void divide(int c, const Plasma::Applet *u1, const Plasma::Applet *u2);
    bool validate(const Group &c) const;
    void balance();
    void join();

    QList<Group> m_groups;
    StandardDesktop *m_desktop;
};



#endif
