/*
 *   Copyright 2008 Vladimir Kuznetsov <ks.vladimir@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef EMBED_WIN_HEADER
#define EMBED_WIN_HEADER

#include <Plasma/Applet>
#include <QX11EmbedContainer>
#include <QPointer>

class QGraphicsLinearLayout;
class QGraphicsProxyWidget;
class QGraphicsView;
class QLabel;

class EmbedWin: public Plasma::Applet
{
    Q_OBJECT
    public:
        EmbedWin(QObject *parent, const QVariantList &args);
        ~EmbedWin();

        void init();

        QList<QAction*> contextualActions();

    protected slots:
        void linkActivated(const QString& link);
        void doEmbed();
        void doDiscard();

        void containerError(QX11EmbedContainer::Error);
        void containerClientIsEmbedded();
        void containerClientClosed();

        void updateWidgetGeometry();

    protected:
        void embedCurrentWindow();
        bool eventFilter(QObject *obj, QEvent *event);

        QGraphicsLinearLayout *m_layout;
        QGraphicsProxyWidget *m_proxy;
        QLabel *m_label;
        QWidget *m_grabber;

        WId m_window;
        bool m_reparenting;
        QPointer<QX11EmbedContainer> m_container;
        QGraphicsView* m_dashboardView;

        QAction* m_actionEmbed;
        QAction* m_actionDiscard;

        static const char* WELCOME_MSG;

        friend class EmbedWinHelper;
};

K_EXPORT_PLASMA_APPLET(embed_win, EmbedWin)

#endif

