/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "audiocapturebin.h"
#include <KDebug>
#include "libqtgstreamer/qgstbin.h"
#include "libqtgstreamer/qgstelement.h"
#include "libqtgstreamer/qgstelementfactory.h"
#include "libqtgstreamer/qgstghostpad.h"
#include "libqtgstreamer/qgstcaps.h"
#include "libqtgstreamer/qgststructure.h"
#include "audioinputdevice.h"

using namespace QtGstreamer;

class AudioCaptureBinPrivate
{
    public:
        AudioInputDevicePtr audioDevice;
        QtGstreamer::QGstElementPtr audioDeviceCapsFilter;
};

AudioCaptureBinPtr AudioCaptureBin::createAudioCaptureBin()
{
    QString driver="autoaudiosrc";
    QString devicePath=QString();
    AudioInputDevicePtr audioDevice = AudioInputDevice::newDevice(driver,devicePath);
    QGstElementPtr capsFilter = QGstElementFactory::make("capsfilter");
    
    if ( !audioDevice || !capsFilter) {
        return AudioCaptureBinPtr();
    }

    AudioCaptureBin *audioCaptureBin = new AudioCaptureBin();
    audioCaptureBin->d->audioDevice = audioDevice;
    audioCaptureBin->d->audioDeviceCapsFilter = capsFilter;
    audioCaptureBin->add(audioDevice);
    audioCaptureBin->add(capsFilter);
    
    QGstCapsPtr caps = QGstCaps::newEmpty();

    QGstStructure yuvStruct("audio/x-raw-yuv");
//    yuvStruct.setValue("width", frameSize.width());

    caps->makeWritable();
    caps->appendStructure(yuvStruct);

    audioCaptureBin->d->audioDeviceCapsFilter->setProperty("caps",caps);
    
    bool success = false;
    success = QtGstreamer::QGstElement::link(audioDevice,capsFilter);
    if (success) {
        QGstPadPtr pad = capsFilter->getStaticPad("src");
        QGstGhostPadPtr ghostPad = QtGstreamer::QGstGhostPad::newGhostPad("src",pad);

        audioCaptureBin->addPad(ghostPad);
    }

    return AudioCaptureBinPtr(audioCaptureBin);
}

AudioCaptureBin::AudioCaptureBin()
    : QGstBin("audio_capture_bin"), d(new AudioCaptureBinPrivate)
{
}

AudioCaptureBin::~AudioCaptureBin()
{
    delete d;
}

void AudioCaptureBin::newDevice(const QString &driver, const QString &devicePath)
{
    this->setState(QGstElement::Null);
    d->audioDevice->setState(QGstElement::Null);

    //remove old
    this->remove(d->audioDevice);

    //create new
    d->audioDevice = AudioInputDevice::newDevice(driver,devicePath);
    this->add(d->audioDevice);
}