/***************************************************************************
 *   Copyright (C) 2009 George Kiagiadakis <kiagiadakis.george@gmail.com>  *
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "audioinputdevice.h"
#include <KDebug>
#include <gst/gst.h>
#include "libqtgstreamer/qgstelementfactory.h"
using namespace QtGstreamer;

class AudioInputDevicePrivate
{
public:
    QGstElementPtr audioDevice;
};

////////////////
////////////////
AudioInputDevice::AudioInputDevice(GstElement *element)
    : QGstElement(element), d(new AudioInputDevicePrivate)
{
    d->audioDevice = QGstElement::fromGstElement(element);
}

AudioInputDevice::~AudioInputDevice()
{
    delete d;
}

AudioInputDevicePtr AudioInputDevice::newDevice(const QString &driver, const QString &devicePath)
{
    QGstElementPtr device = QGstElementFactory::make(driver.toLatin1());
    if (device.isNull()) {
        return AudioInputDevicePtr();
    }
    if (driver != "autoaudiosrc") {
        device->setProperty("device", devicePath.toLatin1());
    }
    return AudioInputDevicePtr(new AudioInputDevice(GST_ELEMENT(device->getNativeObject())));
}

#include "audioinputdevice.moc"