/*
 * This file is part of the KDE project
 * Copyright (C) 2009 Eva Brucherseifer <eva.brucherseifer@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "plasma-killswitch.h"

#include "killswitchitem.h"
#include "configview.h"

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QRect>
#include <QGraphicsGridLayout>

#include <KConfigDialog>

#include <Plasma/Label>
#include <Plasma/IconWidget>
#include <plasma/svg.h>
#include <plasma/theme.h>
 
#include <solid/device.h>

#include <kdebug.h>


PlasmaKillSwitch::PlasmaKillSwitch(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_svg(this),
    m_icon("document"), 
    m_rfkillmgr(this)
{
    m_svg.setImagePath("widgets/background");
    // this will get us the standard applet background, for free!
    setBackgroundHints(DefaultBackground);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    //resize(120, 200);
    //resize(120, 300);
    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
}
 
 
PlasmaKillSwitch::~PlasmaKillSwitch()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
    }
}
 
void PlasmaKillSwitch::init()
{
    QStringList rfkill_uids;
    rfkill_uids = m_rfkillmgr.devicesFromQuery("");
    
    KillSwitchDevice* device;

    foreach (const QString &uid, rfkill_uids) {
        device = dynamic_cast<KillSwitchDevice*>(m_rfkillmgr.createDevice(uid));
        if (device->type() != "unknown")
        {
           m_rfkilldevices.append(device);
           kDebug() << uid << endl;
        }
    }
    buildLayout();
} 

void PlasmaKillSwitch::createConfigurationInterface(KConfigDialog *parent)
{
    QWidget *widget = new ConfigView(parent, &m_rfkillmgr);
    parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
    parent->addPage(widget, parent->windowTitle(), Applet::icon());
    connect(parent, SIGNAL(applyClicked()), widget, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), widget, SLOT(configAccepted()));

    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
}


void PlasmaKillSwitch::configAccepted()
{
    buildLayout();
}


void PlasmaKillSwitch::buildLayout()
{
    // Remove and delete all currently displayed items.
    while (!m_widgets.isEmpty()) {
        QGraphicsWidget *w = m_widgets.takeFirst();
        w->hide();
        delete w;
    }

    // get config
    QStringList inactiveSwitches;
    QStringList switchOrder;

    KSharedConfigPtr configPtr = KSharedConfig::openConfig( "plasma_applet_killswitchrc" );
    KConfigGroup grp( configPtr, QString() );
    if ( grp.hasKey( "InactiveSwitches" ) ) 
        inactiveSwitches = grp.readEntry( "InactiveSwitches", QStringList() );
    if ( grp.hasKey( "SwitchOrder" ) ) 
        switchOrder = grp.readEntry( "SwitchOrder", QStringList() );
    
    QGraphicsGridLayout *layout = new QGraphicsGridLayout();
    setLayout(layout);
    KillSwitchItem* item;

    foreach (const QString &uid, inactiveSwitches) {
        switchOrder.removeAll(uid);
    }
   
    QString devicename; 
    int row, unordered;
    unordered = 0;
    if (!m_rfkilldevices.empty()) 
    {
        for (int i = 0; i < m_rfkilldevices.size(); ++i) 
        {
            devicename = m_rfkilldevices[i]->name();
            if ( !inactiveSwitches.contains(devicename))
            {
                row = switchOrder.indexOf(devicename);
                if (row<0)
                {
                    row = switchOrder.size() + unordered;
                    unordered++;
                }

                item = new KillSwitchItem(m_rfkilldevices[i],this);
                layout->addItem(item, row, 0);
                m_widgets.append(item);
            }
        }
    }
    else 
    {
        Plasma::Label *l = new Plasma::Label(this);
        l->setText(i18n("No switches found"));
        layout->addItem(l, 0, 0);
        //m_widgets.append(l);
    }
/*
    QSizeF size = item->size();
    //size.setHeight(size.height()*m_widgets.size());
    size.setHeight(100*(m_widgets.size()));
    size.setWidth(120);
    resize(size);
*/

/*
    update();
    layout->invalidate();

    QSizeF size = layout->preferredSize();
    size *= 2;
    resize(size);*/
}

 
#include "plasma-killswitch.moc"                         

