/*****************************************************************************
 *   Copyright (C) 2008 by Ben Cooksley <ben@eclipse.endoftheinternet.org>   *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .          *
 ****************************************************************************/

#ifndef KUICKQUIZ_H
#define KUICKQUIZ_H

#include <QString>

#include <Plasma/PopupApplet>
#include <Plasma/Svg>

#include "quizsplash.h"
#include "quizquestion.h"
#include "quizfinish.h"
#include "ui_uiMain.h"
#include "ui_uiSetPassword.h"


class KuickQuiz : public Plasma::PopupApplet
{
    Q_OBJECT
public:
    KuickQuiz(QObject *parent, const QVariantList &args);

    void init();
    void createConfigurationInterface(KConfigDialog *parent);
    QGraphicsWidget * graphicsWidget();

private:
    /// In Panel variables
    QGraphicsWidget * mainWidget;
    QWidget * configMain;
    QWidget * passwordChange;

    /// Display applets
    QuizSplash * splash;
    QuizQuestion * question;
    QuizFinish * finish;

    /// Data on quiz
    QString questionsAndAnswers[100][5];
    int questionAnswers[100];
    QMap<int, QString> results;
    QString quizName;
    int totalQuestions;
    int userScore;
    int questionsDone;
    QString password;

    /// Configuration variables
    Ui::uiMain uiMain;
    Ui::uiSetPassword uiSetPassword;
    bool addingQuestion;

private slots:
    /// Generic slots
    void updateModuleGeometry();

    /// Configuration slots - quiz configuration
    void configAuthenticate();
    void configSetAuthenticated(bool authenticated);
    void configAddQuestion();
    void configEditQuestion();
    void configDelQuestion();
    void configEnableQuestionManagement();
    void configClearScores();
    void configSaveQuestion();
    void configCancelQuestionChanges();
    void configAccept();
    int configActiveQuestion();
    void configPassword();
    void configSetPassword();

    /// Configuration slots - plasmoid configuration
    void configExportQuiz();
    void configImportQuiz();

    /// Applet splash slots
    void splashActive();
    void refreshSplash();

    /// Running quiz slots
    void startQuiz();
    void abortQuiz();
    void nextQuestion();

    /// Post quiz slots
    void insertResults();
    void finishPrepare();

};

K_EXPORT_PLASMA_APPLET(kuickquiz, KuickQuiz)

#endif
