/***************************************************************************
 *   Copyright (C) 2007 by Robert Knight <robertknight@gmail.com>          *
 *   Copyright (C) 2008 by Alexis Ménard <darktears31@gmail.com>           *
 *   Copyright (C) 2009 by Casper Boemann <cbo@boemann.dk>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef TASKGROUPITEM_H
#define TASKGROUPITEM_H

#include "abstracttaskitem.h"
#include "windowtaskitem.h"
// Own
#include <taskmanager/taskmanager.h>
#include "tasksmenu.h"
#include "tasks.h"
#include <QMap>
#include <QHash>

using TaskManager::TaskGroup;
using TaskManager::GroupPtr;
using TaskManager::TaskItem;
using TaskManager::AbstractGroupableItem;


typedef QMap<int, AbstractTaskItem*> Order;

/**
 * A task item for a TaskGroup. It can be displayed collapsed as single item or expanded as group.
 */
class TaskGroupItem : public AbstractTaskItem
{
    Q_OBJECT

public:
    /** Constructs a new representation for a taskgroup. */
    TaskGroupItem(QGraphicsWidget *parent, Tasks *applet);

    /** Destructor */
    virtual ~TaskGroupItem();

    /** Sets the group represented by this task. */
    void setGroup(TaskManager::GroupPtr);

    /** Returns the group represented by this task. */
    TaskManager::GroupPtr group() const;

    virtual void close();

    QList <AbstractTaskItem*> memberList() const;
    AbstractTaskItem * activeSubTask();

    virtual bool isWindowItem() const;
    virtual bool isActive() const;

    /** Returns Direct Member group if the passed item is in a subgroup */
    AbstractTaskItem *directMember(AbstractTaskItem *);

    AbstractTaskItem *taskItemForWId(WId id);

    int indexOf (AbstractTaskItem *task);

    int optimumCapacity();

    AbstractTaskItem* abstractItem(AbstractGroupableItem *);

    void setAdditionalMimeData(QMimeData* mimeData);
    void publishIconGeometry() const;
    void publishIconGeometry(const QRect &rect) const;

signals:
    /** Emitted when a window is selected for activation, minimization, iconification */
    void groupSelected(TaskGroupItem *);
    void sizeHintChanged(Qt::SizeHint);
    /** informs the LayoutWidget about changes */
    void changed();

public slots:
    virtual void activate();
    /**    TaskGroupItem*  groupItem(GroupPtr);

    *   Reload all tasks
    */
    void reload();

    void updatePreferredSize();

protected:
    virtual void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    virtual void paint(QPainter *painter,const QStyleOptionGraphicsItem *option,QWidget *widget);

    virtual void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
    virtual void dropEvent(QGraphicsSceneDragDropEvent *event);
    virtual bool eventFilter(QObject *watched, QEvent *event);
    void handleDroppedId(WId id, AbstractTaskItem *targetTask, QGraphicsSceneDragDropEvent *event);

    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);

public slots:
    void updateActive(AbstractTaskItem *);

protected slots:
    virtual void wheelEvent(QGraphicsSceneWheelEvent *event);

private slots:
    void updateTask(::TaskManager::TaskChanges changes);
    void startNewTask();

    /** Stay informed about changes in group */
    void itemAdded(AbstractGroupableItem *);
    void itemRemoved(AbstractGroupableItem *);

    /** Update to new position*/
    void itemPositionChanged(AbstractGroupableItem *);

    void popupMenu();
    void popdownMenu();
    
    void mouseHoveringMenu(bool hovering);

private:
    AbstractTaskItem* createAbstractItem(AbstractGroupableItem * groupableItem);

    void removeItem(AbstractTaskItem *item);

    void layoutTaskItem(AbstractTaskItem* item, const QPointF &pos);
    void setSplitIndex(int position);

    void reviseVisibilityOfMenu();

    int totalSubTasks();
    AbstractTaskItem * selectSubTask(int index);

    GroupPtr m_group;

    QHash<AbstractGroupableItem *, AbstractTaskItem*> m_groupMembers;

    QTimer *m_startNewTaskTimer;
    QTimer *m_popupMenuTimer;
    QTimer *m_popdownMenuTimer;
    QHash <int, Order> m_taskOrder;
    int m_lastActivated;
    int m_activeTaskIndex;
    TasksMenu *m_menu;
    bool m_inMenu;
    bool m_inButton;
};

#endif
