/***************************************************************************
 *   Copyright (C) 2008 by Thomas Moenicke                                 *
 *   tm@php-qt.org                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "meltdown.h"

#include <QApplication>
#include <QImage>
#include <QPaintDevice>
#include <QLabel>
#include <QPixmap>
#include <QPaintEvent>
#include <QPainter>
#include <QX11Info>
#include <QWidget>
#include <QGraphicsItem>
#include <QColor>

#include <KConfigDialog>

#include <plasma/svg.h>

MeltDown::MeltDown(QObject *parent, const QVariantList &args)
  : Plasma::Applet(parent, args),
    m_dialog(0)
{
  setFlags(QGraphicsItem::ItemIsMovable);

  KConfigGroup cg = config();
  m_xPixelSize = cg.readEntry("xsize", 413);
  m_yPixelSize = cg.readEntry("ysize", 307);

  m_theme = new Plasma::Svg(this);
  m_theme->setImagePath("widgets/meltdown");
  m_theme->setContainsMultipleImages(false);
  m_theme->resize(m_xPixelSize,m_yPixelSize);

  // DataEngine
  engine = dataEngine("meltdown");
  engine->connectAllSources(this);
  connect( engine, SIGNAL(newSource(QString)), SLOT(newSource(QString)) );

  constraintsUpdated();
}

MeltDown::~ MeltDown()
{
}

void MeltDown::constraintsUpdated()
{
  prepareGeometryChange();

  if (formFactor() == Plasma::Planar ||
      formFactor() == Plasma::MediaCenter) {
    QSize s = m_theme->size();
    m_bounds = QRect(0, 0, s.width(), s.height());
  } else {
    QFontMetrics fm(QApplication::font());
    m_bounds = QRectF(0 ,0 ,fm.width("email@something.org"), fm.height() * 1.5);
  }

}
void MeltDown::createConfigurationInterface(KConfigDialog *parent)
{
    QWidget *widget = new QWidget();
    ui.setupUi(widget);

    ui.setupUi(widget);
    parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
    connect( m_dialog, SIGNAL(applyClicked()), this, SLOT(configAccepted()) );
    connect( m_dialog, SIGNAL(okClicked()), this, SLOT(configAccepted()) );
}

void MeltDown::paintInterface(QPainter * painter, const QStyleOptionGraphicsItem * option, const QRect &contentsRect)
{
  Q_UNUSED( option )
  Q_UNUSED( contentsRect )

  painter->setRenderHint(QPainter::SmoothPixmapTransform);
  // draw the main background stuff
  m_theme->paint(painter, boundingRect(), "icecream_circle");

  //painter->scale( m_scale );
}

QRectF MeltDown::boundingRect() const
{
  return m_bounds;
}

void MeltDown::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
{
    m_scale = data["scale"].toInt();
    update();
}

void MeltDown::newSource(const QString & source)
{
  engine->connectSource( source, this );
}

#include "meltdown.moc"
