/***********************************************************************************
* Mini Player: Advanced media player for Plasma.
* Copyright (C) 2008 - 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#ifndef MINIPLAYERPLAYLISTIMPORTER_HEADER
#define MINIPLAYERPLAYLISTIMPORTER_HEADER

#include <QObject>
#include <QTextStream>

#include <KIO/Job>
#include <KIO/NetAccess>

enum PlayListType { None = 0, PLS, M3U, XSPF, ASX };

class MiniPlayerApplet;

class MiniPlayerPlayListImporter : public QObject
{
    Q_OBJECT

    public:
        MiniPlayerPlayListImporter(QString playList, KUrl::List urls, bool play, int index, MiniPlayerApplet *parent);

    public slots:
        void importData(KIO::Job *job, const QByteArray &data);
        void importResult(KJob *job);

    protected:
        void addUrls(KUrl::List items, int level = 0);
        void importPlayList(KUrl url, PlayListType type);
        void readM3u(QTextStream &stream);
        void readPls(QTextStream &stream);
        void readXspf(QByteArray &data);
        void readAsx(QByteArray &data);
        void readDirectory(KUrl url, int level = 0);

    private:
        QHash<KJob*, QByteArray> m_remotePlayListsData;
        QHash<KJob*, PlayListType> m_remotePlayListsType;
        QHash<KUrl, QString> m_titles;
        KUrl::List m_tracks;
        QString m_playList;
        int m_imports;
        int m_index;
        bool m_play;

    signals:
        void processedTracks(QString playList, KUrl::List tracks, QHash<KUrl, QString> titles, bool play, int index);
};

#endif
