/***************************************************************************
 *   Copyright (C) 2008 by Matej Svejda <mata@aw-modell.at>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
 
#include "peachy_animator.h"

#include "peachy_applet.h"
#include "peachy_icon.h"

#include <cmath>

#define PI 3.141592653

PeachyAnimator::PeachyAnimator(qreal scaling, PeachyApplet* parent)
	: QObject(parent)
{
	m_minimumScaling = scaling;
}

PeachyAnimator::~PeachyAnimator()
{

}

void PeachyAnimator::setMinimumScaling(qreal scaling)
{
	m_minimumScaling = scaling;
}

void PeachyAnimator::add(PeachyIcon* icon)
{
	icon->setScaling(m_minimumScaling);
	m_icons.append(icon);
	connect(icon, SIGNAL(gotHover(PeachyIcon*, qreal)), this, SLOT(iconGotHover(PeachyIcon*, qreal)));
}

void PeachyAnimator::remove(PeachyIcon* icon)
{
	m_icons.removeAll(icon);
	disconnect(icon, 0, this, 0);
}

void PeachyAnimator::clear()
{
	m_icons.clear();
}

void PeachyAnimator::iconGotHover(PeachyIcon* icon, qreal across)
{
	kDebug() << "icon got hover";
	int activeItemPosition = m_icons.indexOf(icon);
	for(int i = 0; i < m_icons.size(); i++) {
		if(i < activeItemPosition - 3 || i > activeItemPosition + 3) {
			m_icons.at(i)->setScaling(m_minimumScaling);
		} else if(i == activeItemPosition) {
			m_icons.at(i)->setScaling(1);
		} else if(i < activeItemPosition) {
			m_icons.at(i)->setScaling(m_minimumScaling + (1 - m_minimumScaling) * (std::cos((i - activeItemPosition - across + 1) / 3 * PI) + 1) / 2);
		} else {
			m_icons.at(i)->setScaling(m_minimumScaling + (1 - m_minimumScaling) * (std::cos((i - activeItemPosition - across) / 3 * PI) + 1) / 2);
		}
	}
	emit needsRelayout();
}
void PeachyAnimator::reinit()
{
	foreach(PeachyIcon* icon, m_icons) {
		icon->setScaling(m_minimumScaling);
	}
	emit needsRelayout();
}
