/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "animatedlayout.h"
#include "proxylayout.h"

// Qt
#include <QGraphicsWidget>


AnimatedLayout::AnimatedLayout(const Qt::Orientation &orientation,
                               QGraphicsLayoutItem *parent)
    : QGraphicsLinearLayout(orientation, parent)
{




}


AnimatedLayout::~AnimatedLayout()
{



}


void AnimatedLayout::addItem(QGraphicsLayoutItem *item)
{

    insertItem(count(), item);

}


void AnimatedLayout::insertItem(int index, QGraphicsLayoutItem *item)
{

    ProxyLayout *pLayout = new ProxyLayout(static_cast<QGraphicsWidget*>(item));
    QGraphicsLinearLayout::insertItem(index, pLayout);


}


void AnimatedLayout::removeAt(int index)
{

    ProxyLayout *pLayout = static_cast<ProxyLayout*>(QGraphicsLinearLayout::itemAt(index));
    QGraphicsLinearLayout::removeAt(index);
    pLayout->removeAt(0);
    delete pLayout;
    
}


void AnimatedLayout::removeItem(QGraphicsLayoutItem *item)
{

    for (int i = 0; i < count(); i++) {
        if (item == itemAt(i)) {
            removeAt(i);
            break;
        }
    }

}


QGraphicsLayoutItem *AnimatedLayout::itemAt(int index) const
{

    ProxyLayout *pLayout = static_cast<ProxyLayout*>(QGraphicsLinearLayout::itemAt(index));
    return pLayout->itemAt(0);

}


