/***************************************************************************
 *   Copyright 2007 by Marco Martin <notmart@gmail.com>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "svgpaneltest.h"

//QT
#include <QGraphicsSceneDragDropEvent>
#include <QDesktopWidget>
#include <QApplication>

//KDE
#include <KDebug>
#include <KLocale>
#include <KIconLoader>
#include <KMimeType>
#include <KRun>
#include <KSharedConfig>
#include <KMessageBox>
#include <KConfigDialog>
#include <KUrl>

#include <plasma/containment.h>
#include <plasma/svg.h>
#include <plasma/framesvg.h>

namespace Plasma
{

SvgPanelTest::SvgPanelTest(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_SvgFileName(QString()),
      m_dialog(0)
{
    setHasConfigurationInterface(true);

    if (args.count() > 2) {
        m_SvgFileName = args.at(2).toString();
    }
}

SvgPanelTest::~SvgPanelTest()
{
}

void SvgPanelTest::init()
{
    KConfigGroup cg = config();
    if (m_SvgFileName.isEmpty()) {
        m_SvgFileName = cg.readEntry("svgFileName", "widgets/background");
    }
    m_Prefix = cg.readEntry("Prefix", QString());
    m_DrawTop = cg.readEntry("DrawTop", true);
    m_DrawBottom = cg.readEntry("DrawBottom", true);
    m_DrawLeft = cg.readEntry("DrawLeft", true);
    m_DrawRight = cg.readEntry("DrawRight", true);

    m_panel = new FrameSvg(this);
    m_panel->setImagePath(m_SvgFileName);
    m_panel->setElementPrefix(m_Prefix);
    m_panel->setEnabledBorders(Plasma::FrameSvg::AllBorders);
    m_panel->resizeFrame(size());
    updateFlags();
    setBackgroundHints(NoBackground);
    setHasConfigurationInterface(true);
}

Qt::Orientations SvgPanelTest::expandingDirections() const
{
    return 0;
}


void SvgPanelTest::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option,
                            const QRect &contents)
{
    m_panel->paintFrame(painter);
}


void SvgPanelTest::constraintsEvent(Plasma::Constraints constraints)
{
    setBackgroundHints(NoBackground);

    if (constraints & Plasma::SizeConstraint && m_panel) {
        m_panel->resizeFrame(size());
    }

    updateGeometry();
}

void SvgPanelTest::createConfigurationInterface(KConfigDialog *parent)
{

    QWidget *widget = new QWidget;
    ui.setupUi(widget);
    parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
    parent->addPage(widget, parent->windowTitle(), icon());
    connect( parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()) );
    connect( parent, SIGNAL(okClicked()), this, SLOT(configAccepted()) );


    ui.svgFileName->setUrl(m_SvgFileName);
    ui.prefix->setText(m_Prefix);
    ui.drawTop->setChecked(m_DrawTop);
    ui.drawBottom->setChecked(m_DrawBottom);
    ui.drawLeft->setChecked(m_DrawLeft);
    ui.drawRight->setChecked(m_DrawRight);
}

void SvgPanelTest::configAccepted()
{
    KConfigGroup cg = config();
    m_SvgFileName = ui.svgFileName->url().path();
    m_Prefix = ui.prefix->text();

    m_DrawTop = ui.drawTop->isChecked();
    m_DrawBottom = ui.drawBottom->isChecked();
    m_DrawLeft = ui.drawLeft->isChecked();
    m_DrawRight = ui.drawRight->isChecked();

    cg.writeEntry("svgFileName", m_SvgFileName);
    cg.writeEntry("Prefix", m_Prefix);
    cg.writeEntry("DrawTop", m_DrawTop);
    cg.writeEntry("DrawBottom", m_DrawBottom);
    cg.writeEntry("DrawLeft", m_DrawLeft);
    cg.writeEntry("DrawRight", m_DrawRight);

    m_panel->setImagePath(m_SvgFileName);
    m_panel->setElementPrefix(m_Prefix);
    updateFlags();

    update();
    emit configNeedsSaving();
}

void SvgPanelTest::updateFlags()
{
   m_panel->setEnabledBorders(
                FrameSvg::EnabledBorders(
                    (m_DrawTop?FrameSvg::TopBorder:0) |
                    (m_DrawBottom?FrameSvg::BottomBorder:0) |
                    (m_DrawLeft?FrameSvg::LeftBorder:0) |
                    (m_DrawRight?FrameSvg::RightBorder:0)) );
}

} // Plasma namespace

#include "svgpaneltest.moc"

