#include <cstdlib>

#include <QPainter>
#include <QTextEdit>
#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>

#include <KLineEdit>

#include "systemcommand.h"

SystemCommand::SystemCommand(QObject* parent, const QVariantList& args)
: Plasma::Applet(parent, args), m_lineEdit(0), m_textEdit(0),
  m_lineEditProxy(0), m_textEditProxy(0), m_form(0)
{
    setBackgroundHints(DefaultBackground);
    resize(500, 400);
}

SystemCommand::~SystemCommand()
{
    m_lineEditProxy->setWidget(0);
    m_textEditProxy->setWidget(0);
    delete m_textEdit;
    delete m_lineEdit;
}

void SystemCommand::init()
{
    m_lineEditProxy = new QGraphicsProxyWidget(this);
    m_lineEdit = new KLineEdit;
    m_lineEditProxy->setWidget(m_lineEdit);

    m_textEditProxy = new QGraphicsProxyWidget(this);
    m_textEdit = new QTextEdit;
    m_textEdit->setReadOnly(true);
    m_textEditProxy->setWidget(m_textEdit);

    QGraphicsLinearLayout* l = new QGraphicsLinearLayout(Qt::Vertical);
    l->addItem(m_lineEditProxy);
    l->addItem(m_textEditProxy);

    m_form = new QGraphicsProxyWidget(this);
    m_form->setLayout(l);

    m_form->show();
    m_lineEditProxy->show();
    m_textEditProxy->show();

    connect(&m_shell, SIGNAL(readyReadStandardOutput()), this, SLOT(stdout()));
    connect(&m_shell, SIGNAL(readyReadStandardError()), this, SLOT(stderr()));
    connect(&m_shell, SIGNAL(stateChanged(QProcess::ProcessState)),
             this, SLOT(processStateChanged(QProcess::ProcessState)));
    connect(m_lineEdit, SIGNAL(returnPressed()), this, SLOT(run()));
}

void SystemCommand::stdout()
{
    QByteArray out = m_shell.readAllStandardOutput();
    m_textEdit->append(QString::fromLocal8Bit(out));
}

void SystemCommand::stderr()
{
    QByteArray out = m_shell.readAllStandardError();
    m_textEdit->append(QString::fromLocal8Bit(out));
}

void SystemCommand::run()
{
    QString shell = std::getenv("SHELL");
    QString cmd = m_lineEdit->text();
    QString command = QString("%1 -c \"%2\"").arg(shell).arg(cmd);
    m_textEdit->clear();
    m_lineEdit->clear();
    m_shell.start(command);
}

void SystemCommand::processStateChanged(QProcess::ProcessState state)
{
    switch (state) {
        case QProcess::Starting:
        case QProcess::Running:
            m_lineEdit->setEnabled(false);
            break;
        case QProcess::NotRunning:
            m_lineEdit->setEnabled(true);
            m_lineEdit->setFocus();
            break;
    }
}

void SystemCommand::paintInterface(QPainter* painter,
                                    const QStyleOptionGraphicsItem* option,
                                    const QRect& contentsRect)
{
    Q_UNUSED(painter);
    Q_UNUSED(option);
    m_form->setPos(contentsRect.topLeft());
    m_form->resize(contentsRect.width(), contentsRect.height());
}

#include "systemcommand.moc"
