
#include "panel.h"

#include <limits>

#include <QApplication>
#include <QGraphicsLinearLayout>
#include <QPainter>
#include <QBitmap>
#include <QDesktopWidget>
#include <QGridLayout>
#include <QLabel>
#include <QComboBox>
#include <QAction>
#include <QGraphicsLayout>
#include <QGraphicsSceneDragDropEvent>
#include <iostream>

#include <KDebug>
#include <KIcon>
#include <KDialog>
#include <KIntNumInput>
#include <KMessageBox>

#include <Plasma/Corona>
#include <Plasma/FrameSvg>
#include <Plasma/Theme>
#include <Plasma/View>
#include <Plasma/PaintUtils>



#include <assert.h>

namespace welcome_plasmoid {
class Spacer : public QGraphicsWidget
{
public:
    Spacer(QGraphicsWidget *parent)
         : QGraphicsWidget(parent)
    {
        setAcceptDrops(true);
    }

    ~Spacer()
    {}

    Panel *panel;

protected:
    void dropEvent(QGraphicsSceneDragDropEvent *event)
    {
        event->setPos(mapToParent(event->pos()));
        panel->dropEvent(event);
    }

    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget * widget = 0)
    {
        Q_UNUSED(option)
        Q_UNUSED(widget)

        //TODO: make this a pretty gradient?
        painter->setRenderHint(QPainter::Antialiasing);
        QPainterPath p = Plasma::PaintUtils::roundedRectangle(contentsRect().adjusted(1, 1, -2, -2), 4);
        QColor c = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
        c.setAlphaF(0.3);

        painter->fillPath(p, c);
    }
};

Panel::Panel(QObject *parent, const QVariantList &args)
    : Containment(parent, args),
      m_configureAction(0),
      m_addPanelAction(0),
      m_currentSize(QSize(100, 35)),
      m_maskDirty(true),
      m_spacerIndex(-1),
      m_spacer(0)
{
//    m_background = new Plasma::FrameSvg(this);
//    m_background->setImagePath("widgets/panel-background");
//    m_background->setEnabledBorders(Plasma::FrameSvg::AllBorders);

//    connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(backgroundChanged()));
    setZValue(150);

    resize(m_currentSize);
    setMinimumSize(m_currentSize);
    setMaximumSize(m_currentSize);

    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
    connect(this, SIGNAL(appletAdded(Plasma::Applet*,QPointF)),
            this, SLOT(layoutApplet(Plasma::Applet*,QPointF)));
    connect(this, SIGNAL(appletRemoved(Plasma::Applet*)),
            this, SLOT(appletRemoved(Plasma::Applet*)));

    setContainmentType(Containment::CustomPanelContainment);
    updateConstraints(Plasma::FormFactorConstraint);
    Containment::init();
   setAttribute(Qt::WA_NoSystemBackground);
}
Panel::Panel(QGraphicsWidget* parent, QString identifier, uint& appletid)
    : Containment(parent, identifier, appletid),
      m_configureAction(0),
      m_addPanelAction(0),
      m_currentSize(QSize(50, 35)),
      m_maskDirty(true),
      m_spacerIndex(-1),
      m_spacer(0)
{
setAttribute(Qt::WA_NoSystemBackground);
   // setParentItem(parent);
//    m_background = new Plasma::FrameSvg(this);
//    m_background->setImagePath("widgets/panel-background");
//    m_background->setEnabledBorders(Plasma::FrameSvg::AllBorders);
//    connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(backgroundChanged()));
    setZValue(150);


    resize(m_currentSize);
    setMinimumSize(m_currentSize);
    setMaximumSize(m_currentSize);

    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
    connect(this, SIGNAL(appletAdded(Plasma::Applet*,QPointF)),
            this, SLOT(layoutApplet(Plasma::Applet*,QPointF)));
    connect(this, SIGNAL(appletRemoved(Plasma::Applet*)),
            this, SLOT(appletRemoved(Plasma::Applet*)));

    setContainmentType(Containment::CustomPanelContainment);
    updateConstraints(Plasma::FormFactorConstraint);
    flushPendingConstraintsEvents();
    Containment::init();
}
Panel::~Panel()
{
 setParentItem(NULL);
 setParent(NULL);
}
bool Panel::isContainment() const { return true; }
void Panel::init()
{



    //FIXME: This should be enabled, but in that case proxywidgets won't get rendered
    //setFlag(ItemClipsChildrenToShape, true);

    KConfigGroup cg = config("Configuration");

    m_currentSize = cg.readEntry("minimumSize", m_currentSize);
    if (formFactor() == Plasma::Vertical) {
        m_currentSize.expandedTo(QSize(0, 35));
    } else {
        m_currentSize.expandedTo(QSize(35, 0));
    }
	setLocation(Plasma::BottomEdge);
    setMinimumSize(cg.readEntry("minimumSize", m_currentSize));
    setMaximumSize(cg.readEntry("maximumSize", m_currentSize));
//    setDrawWallpaper(false);
}

QList<QAction*> Panel::contextualActions()
{
return QList<QAction*> ();
    if (!m_configureAction) {
        m_configureAction = new QAction(i18n("Panel Settings"), this);
        m_configureAction->setIcon(KIcon("configure"));
        connect(m_configureAction, SIGNAL(triggered()), this, SIGNAL(toolBoxToggled()));

        m_addPanelAction = new QAction(i18n("Add Panel"), this);
        connect(m_addPanelAction, SIGNAL(triggered(bool)), this, SLOT(addPanel()));
        m_addPanelAction->setIcon(KIcon("list-add"));
        constraintsEvent(Plasma::ImmutableConstraint);
    }

    QList<QAction*> actions;
    actions << action("add widgets") << m_addPanelAction << action("lock widgets") << m_configureAction << action("remove");
    return actions;
}

void Panel::backgroundChanged()
{
    constraintsEvent(Plasma::LocationConstraint);
}

void Panel::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
{
    // this gets called whenever an applet is added, and we add it to our layout
    QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(layout());

    if (!lay) {
	kDebug() << "No layout, not adding applet!\n";
        return;
    }
	kDebug() << "Layout found, adding applet!\n";

    Plasma::FormFactor f = formFactor();
    int insertIndex = -1;

    //Enlarge the panel if possible and needed
    QSizeF appletHint = applet->preferredSize();
    QSizeF panelHint = layout()->preferredSize();
    if (f == Plasma::Horizontal) {
        if (panelHint.width() + appletHint.width() > size().width()) {
            resize(panelHint.width() + appletHint.width(), size().height());
        }
    } else {
        if (panelHint.height() + appletHint.height() > size().height()) {
            resize(size().width(), panelHint.height() + appletHint.height());
        }
    }
    layout()->setMaximumSize(size());

    //if pos is (-1,-1) insert at the end of the panel
    if (pos != QPoint(-1, -1)) {
        for (int i = 0; i < lay->count(); ++i) {
            QRectF siblingGeometry = lay->itemAt(i)->geometry();
            if (f == Plasma::Horizontal) {
                qreal middle = (siblingGeometry.left() + siblingGeometry.right()) / 2.0;
                if (pos.x() < middle) {
                    insertIndex = i;
                    break;
                } else if (pos.x() <= siblingGeometry.right()) {
                    insertIndex = i + 1;
                    break;
                }
            } else { // Plasma::Vertical
                qreal middle = (siblingGeometry.top() + siblingGeometry.bottom()) / 2.0;
                if (pos.y() < middle) {
                    insertIndex = i;
                    break;
                } else if (pos.y() <= siblingGeometry.bottom()) {
                    insertIndex = i + 1;
                    break;
                }
            }
        }
    }

    if (insertIndex == -1) {
        lay->addItem(applet);
    } else {
        lay->insertItem(insertIndex, applet);
    }

    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
}

void Panel::appletRemoved(Plasma::Applet* applet)
{
    QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(layout());
    lay->removeItem(applet);

    //shrink the panel if possible
    if (formFactor() == Plasma::Horizontal) {
        resize(size().width() - applet->size().width(), size().height());
    } else {
        resize(size().width(), size().height() - applet->size().height());
    }
    layout()->setMaximumSize(size());
}

void Panel::updateSize()
{
    Plasma::Applet *applet = qobject_cast<Plasma::Applet *>(sender());

    if (applet) {
        if (formFactor() == Plasma::Horizontal) {
            const int delta = applet->preferredWidth() - applet->size().width();
            //setting the preferred width when delta = 0 and preferredWidth() < minimumWidth()
            // leads to the same thing as setPreferredWidth(minimumWidth())
            if (delta != 0) {
                setPreferredWidth(preferredWidth() + delta);
            }
        } else if (formFactor() == Plasma::Vertical) {
            const int delta = applet->preferredHeight() - applet->size().height();
            if (delta != 0) {
                setPreferredHeight(preferredHeight() + delta);
            }
        }

        resize(preferredSize());
    }
}

void Panel::addPanel()
{
    if (corona()) {
        Containment* panel = corona()->addContainment("panel");
        panel->showConfigurationInterface();

        panel->setScreen(screen());

        QList<Plasma::Location> freeEdges = corona()->freeEdges(screen());
        kDebug() << freeEdges;
        Plasma::Location destination;
        if (freeEdges.contains(Plasma::TopEdge)) {
            destination = Plasma::TopEdge;
        } else if (freeEdges.contains(Plasma::BottomEdge)) {
            destination = Plasma::BottomEdge;
        } else if (freeEdges.contains(Plasma::LeftEdge)) {
            destination = Plasma::LeftEdge;
        } else if (freeEdges.contains(Plasma::RightEdge)) {
            destination = Plasma::RightEdge;
        } else destination = Plasma::TopEdge;

        panel->setLocation(destination);

        // trigger an instant layout so we immediately have a proper geometry
        // rather than waiting around for the event loop
        panel->updateConstraints(Plasma::StartupCompletedConstraint);
        panel->flushPendingConstraintsEvents();

//        if (destination == Plasma::LeftEdge ||
//            destination == Plasma::RightEdge) {
//            panel->setMinimumSize(10, 35);
//            panel->setMaximumSize(35,Kephal::ScreenUtils::screenSize(screen()).height());
//            panel->resize(QSize(35, Kephal::ScreenUtils::screenSize(screen()).height()));
//        }
    }
}

void Panel::updateBorders(const QRect &geom)
{
    Plasma::Location loc = location();
    Plasma::FrameSvg::EnabledBorders enabledBorders = Plasma::FrameSvg::AllBorders;

    int s = screen();
    //kDebug() << loc << s << formFactor() << geometry();

    qreal topHeight = 4;//m_background->marginSize(Plasma::TopMargin);
    qreal bottomHeight = 4;//m_background->marginSize(Plasma::BottomMargin);
    qreal leftWidth = 4;//m_background->marginSize(Plasma::LeftMargin);
    qreal rightWidth = 4;//m_background->marginSize(Plasma::RightMargin);

    //remove unwanted borders
    if (s < 0) {
        // do nothing in this case, we want all the borders
    } else if (loc == Plasma::BottomEdge || loc == Plasma::TopEdge) {
    //    QRect r = Kephal::ScreenUtils::screenGeometry(s);
	QRect r = geometry().toRect();
        if (loc == Plasma::BottomEdge) {
            enabledBorders ^= Plasma::FrameSvg::BottomBorder;
            bottomHeight = 0;
        } else {
            enabledBorders ^= Plasma::FrameSvg::TopBorder;
            topHeight = 0;
        }

        if (geom.x() <= r.x()) {
            enabledBorders ^= Plasma::FrameSvg::LeftBorder;
            leftWidth = 0;
        }
        if (geom.right() >= r.right()) {
            enabledBorders ^= Plasma::FrameSvg::RightBorder;
            rightWidth = 0;
        }

        //kDebug() << "top/bottom: Width:" << width << ", height:" << height;
    } else if (loc == Plasma::LeftEdge || loc == Plasma::RightEdge) {
//        QRect r = Kephal::ScreenUtils::screenGeometry(s);
	QRect r = geometry().toRect();
        if (loc == Plasma::RightEdge) {
            enabledBorders ^= Plasma::FrameSvg::RightBorder;
            rightWidth = 0;
        } else {
            enabledBorders ^= Plasma::FrameSvg::LeftBorder;
            leftWidth = 0;
        }
        if (geom.y() <= r.y()) {
            enabledBorders ^= Plasma::FrameSvg::TopBorder;
            topHeight = 0;
        }
        if (geom.bottom() >= r.bottom()) {
            enabledBorders ^= Plasma::FrameSvg::BottomBorder;
            bottomHeight = 0;
        }

        //kDebug() << "left/right: Width:" << width << ", height:" << height;
    } else {
        kDebug() << "no location!?";
    }

    //activate borders and fetch sizes again
//    m_background->setEnabledBorders(enabledBorders);
//    m_background->getMargins(leftWidth, topHeight, rightWidth, bottomHeight);

    //calculation of extra margins has to be done after getMargins
    const QGraphicsItem *box = toolBoxItem();
    if (box && immutability() == Plasma::Mutable) {
        QSizeF s = box->boundingRect().size();
        if (formFactor() == Plasma::Vertical) {
            //hardcoded extra margin for the toolbox right now
            bottomHeight += s.height();;
            //Default to horizontal for now
        } else {
            if (QApplication::layoutDirection() == Qt::RightToLeft) {
                leftWidth += s.width();
            } else {
                rightWidth += s.width();
            }
        }
    }

    //invalidate the layout and set again
    if (layout()) {
        switch (location()) {
        case Plasma::LeftEdge:
            rightWidth = qMin(rightWidth, qMax(qreal(1), size().width() - KIconLoader::SizeMedium));
            break;
        case Plasma::RightEdge:
            leftWidth = qMin(leftWidth, qMax(qreal(1), size().width() - KIconLoader::SizeMedium));
            break;
        case Plasma::TopEdge:
            bottomHeight = qMin(bottomHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
            break;
        case Plasma::BottomEdge:
            topHeight = qMin(topHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
            break;
        default:
            break;
        }
        layout()->setContentsMargins(leftWidth, topHeight, rightWidth, bottomHeight);
        layout()->invalidate();
    }

    update();
}

void Panel::constraintsEvent(Plasma::Constraints constraints)
{
    kDebug() << "constraints updated with" << constraints << "!!!!!!";

    m_maskDirty = true;

    if (constraints & Plasma::FormFactorConstraint) {
        Plasma::FormFactor form = formFactor();
        Qt::Orientation layoutDirection = form == Plasma::Vertical ? Qt::Vertical : Qt::Horizontal;
        // create our layout!
        if (layout()) {
            QGraphicsLayout *lay = layout();
            QGraphicsLinearLayout * linearLay = dynamic_cast<QGraphicsLinearLayout *>(lay);
            if (linearLay) {
                linearLay->setOrientation(layoutDirection);
            }
            linearLay->setMaximumSize(size());
        } else {
            QGraphicsLinearLayout *lay = new QGraphicsLinearLayout(this);
            lay->setOrientation(layoutDirection);
            lay->setContentsMargins(0, 0, 0, 0);
            lay->setSpacing(4);
            lay->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
            setLayout(lay);
            updateBorders(geometry().toRect());

            foreach (Applet *applet, applets()) {
                lay->addItem(applet);
            }

            lay->setMaximumSize(size());
        }
    }

    //we need to know if the width or height is 100%
    if (constraints & Plasma::LocationConstraint || constraints & Plasma::SizeConstraint) {
        m_currentSize = geometry().toRect().size();//parentItem()->boundingRect().toRect().size();
//        QRectF screenRect = screen() >= 0 ? Kephal::ScreenUtils::screenGeometry(screen()) :
  //          geometry();
	QRectF screenRect = geometry();//parentItem()->boundingRect();//geometry();
        if ((formFactor() == Plasma::Horizontal && m_currentSize.width() >= screenRect.width()) ||
                (formFactor() == Plasma::Vertical && m_currentSize.height() >= screenRect.height())) {
//            m_background->setElementPrefix(location());
        } else {
            switch (location()) {
            case Plasma::LeftEdge:
                //this call will automatically fallback at no prefix if the element isn't available
//                m_background->setElementPrefix("west-mini");
                break;
            case Plasma::RightEdge:
//                m_background->setElementPrefix("east-mini");
                break;
            case Plasma::TopEdge:
//                m_background->setElementPrefix("north-mini");
                break;
            case Plasma::BottomEdge:
            default:
//                m_background->setElementPrefix("south-mini");
                break;
            }
        }

//        m_background->resizeFrame(m_currentSize);
    }

    //FIXME: this seems the only way to correctly resize the layout the first time when the
    // saved panel size is less than the default is to setting a maximum size.
    // this shouldn't happen. maybe even a qgraphicslayout bug?
    if (layout() && (constraints & Plasma::SizeConstraint)) {
        layout()->setMaximumSize(size());
    }

    if (constraints & Plasma::LocationConstraint) {
        setFormFactorFromLocation(location());
    }

    if (constraints & Plasma::ImmutableConstraint) {
        bool unlocked = immutability() == Plasma::Mutable;

        if (m_addPanelAction) {
            m_addPanelAction->setEnabled(unlocked);
            m_addPanelAction->setVisible(unlocked);
        }

        if (m_configureAction) {
            m_configureAction->setEnabled(unlocked);
            m_configureAction->setVisible(unlocked);
        }

        QGraphicsView *panelView = view();
        if (panelView) {
            updateBorders(panelView->geometry());
        } else updateBorders(geometry().toRect());
    }
}

void Panel::saveState(KConfigGroup &config) const
{
    config.writeEntry("minimumSize", minimumSize());
    config.writeEntry("maximumSize", maximumSize());
}

void Panel::themeUpdated()
{
    //if the theme is changed all the calculations needs to be done again
    //and resize based on the change in the theme bordersize

    qreal oldLeftWidth;
    qreal newLeftWidth = 4;
    qreal oldTopHeight;
    qreal newTopHeight = 4;
    qreal oldRightWidth;
    qreal newRightWidth = 4;
    qreal oldBottomHeight;
    qreal newBottomHeight = 4;

    layout()->getContentsMargins(&oldLeftWidth, &oldTopHeight, &oldRightWidth, &oldBottomHeight);
//    m_background->getMargins(newLeftWidth, newTopHeight, newRightWidth, newBottomHeight);

    QSize newSize(size().width()-(oldLeftWidth - newLeftWidth)-(oldRightWidth - newRightWidth),
           size().height()-(oldTopHeight - newTopHeight)-(oldBottomHeight - newBottomHeight));

    resize(newSize);

    if (formFactor() == Plasma::Vertical) {
        setMaximumWidth(newSize.width());
        setMinimumWidth(newSize.width());
    } else {
        setMaximumHeight(newSize.height());
        setMinimumHeight(newSize.height());
    }

    updateBorders(geometry().toRect());
}

void Panel::paintInterface(QPainter *painter,
                           const QStyleOptionGraphicsItem *option,
                           const QRect& contentsRect)
{
    Q_UNUSED(contentsRect)
    //FIXME: this background drawing is bad and ugly =)
    // draw the background untransformed (saves lots of per-pixel-math)
    painter->save();
    painter->resetTransform();

    const Containment::StyleOption *containmentOpt = qstyleoption_cast<const Containment::StyleOption *>(option);

    QRect viewGeom;
    if (containmentOpt && containmentOpt->view) {
        viewGeom = containmentOpt->view->geometry();
    }

    if (m_maskDirty || m_lastViewGeom != viewGeom) {
        m_maskDirty = false;
        m_lastViewGeom = viewGeom;

        updateBorders(viewGeom);
//        if (containmentOpt && containmentOpt->view) {
//            containmentOpt->view->setMask(m_background->mask());
//        }
    }

    // blit the background (saves all the per-pixel-products that blending does)
    painter->setCompositionMode(QPainter::CompositionMode_Source);
    painter->setRenderHint(QPainter::Antialiasing);

//    m_background->paintFrame(painter, option->exposedRect);

    // restore transformation and composition mode
    painter->restore();
}

void Panel::setFormFactorFromLocation(Plasma::Location loc) {
    switch (loc) {
        case Plasma::BottomEdge:
        case Plasma::TopEdge:
            //kDebug() << "setting horizontal form factor";
            setFormFactor(Plasma::Horizontal);
            break;
        case Plasma::RightEdge:
        case Plasma::LeftEdge:
            //kDebug() << "setting vertical form factor";
            setFormFactor(Plasma::Vertical);
            break;
        case Plasma::Floating:
            //TODO: implement a form factor for floating panels
            kDebug() << "Floating is unimplemented.";
		setFormFactor(Plasma::Horizontal);
            break;
        default:
            kDebug() << "invalid location!!";
    }
}

Plasma::Applet *Panel::addApplet(const QString &name, const QVariantList &args,
                                       const QRectF &appletGeometry, uint id, bool delayInit)
 {
	kDebug() << "Applet added!\n"; 
     if (!delayInit && immutability() != Plasma::Mutable) {
         kDebug() << "addApplet for" << name << "requested, but we're currently immutable!";
        return 0;
    }
 
     QGraphicsView *v = view();
     if (v) {
         v->setCursor(Qt::BusyCursor);
     }
 
     Applet *applet = Applet::load(name, id, args);
     if (v) {
         v->unsetCursor();
     }
 
     if (!applet) {
         kDebug() << "Applet" << name << "could not be loaded.";
         applet = new Applet(0, QString(), id);
//         applet->setFailedToLaunch(true, i18n("Could not find requested component: %1", name));
     }
 
     //kDebug() << applet->name() << "sizehint:" << applet->sizeHint() << "geometry:" << applet->geometry();
 
     addApplet(applet, appletGeometry.topLeft(), delayInit);
     return applet;
}
void Panel::addApplet(Applet *applet, const QPointF &pos, bool delayInit)
 {
	Plasma::Containment::addApplet(applet, pos, false);
	applet->updateConstraints(Plasma::StartupCompletedConstraint);
	applet->updateConstraints(Plasma::AllConstraints);
	applet->flushPendingConstraintsEvents();
         applet->setParentItem(this);

	/*
     if (!applet) {
         kDebug() << "adding null applet!?!";
         return;
     }
 
     if (d->applets.contains(applet)) {
         kDebug() << "already have this applet!";
     }
 
     Containment *currentContainment = applet->containment();
 
//     if (d->type == PanelContainment) {
         //panels don't want backgrounds, which is important when setting geometry
         setBackgroundHints(NoBackground);
//     }
 
     if (currentContainment && currentContainment != this) {
         emit currentContainment->appletRemoved(applet);
         disconnect(applet, 0, currentContainment, 0);
         applet->removeSceneEventFilter(currentContainment);
         KConfigGroup oldConfig = applet->config();
         currentContainment->d->applets.removeAll(applet);
         if (currentContainment->d->handles.contains(applet)) {
             currentContainment->d->handles.remove(applet);
         }

         applet->setParentItem(this);
 
         // now move the old config to the new location
         //FIXME: this doesn't seem to get the actual main config group containing plugin=, etc
         KConfigGroup c = config().group("Applets").group(QString::number(applet->id()));
         oldConfig.reparent(&c);
         applet->d->resetConfigurationObject();
 
         disconnect(applet, SIGNAL(activate()), currentContainment, SIGNAL(activate()));
     } else {
         applet->setParentItem(this);
     }
 
     d->applets << applet;
 
     connect(applet, SIGNAL(configNeedsSaving()), this, SIGNAL(configNeedsSaving()));
     connect(applet, SIGNAL(releaseVisualFocus()), this, SIGNAL(releaseVisualFocus()));
     connect(applet, SIGNAL(appletDestroyed(Plasma::Applet*)), this, SLOT(appletDestroyed(Plasma::Applet*)));
     connect(applet, SIGNAL(activate()), this, SIGNAL(activate()));

     if (pos != QPointF(-1, -1)) {
         applet->setPos(pos);
     }
 
     if (delayInit || currentContainment) {
         if (d->type == DesktopContainment) {
             applet->installSceneEventFilter(this);
             //applet->setWindowFlags(Qt::Window);
         }
     } else {
         applet->init();
         Plasma::Animator::self()->animateItem(applet, Animator::AppearAnimation);
     }
 
     applet->updateConstraints(Plasma::AllConstraints);
 
     if (!delayInit) {
         applet->flushPendingConstraintsEvents();
     }
 
     emit appletAdded(applet, pos);
 
     if (!currentContainment) {
         applet->updateConstraints(Plasma::StartupCompletedConstraint);
         if (!delayInit) {
             applet->flushPendingConstraintsEvents();
         }
    }
 
//    if (!delayInit) {
//         applet->d->scheduleModificationNotification();
//     }
*/
 }



void Panel::showDropZone(const QPoint pos)
{
    if (!scene()) {
        return;
    }

    // if the drop isn't happening on the outer edges and is instead
    // actually poised over an applet, ignore it
    if (((formFactor() == Plasma::Vertical && pos.y() > 1 && pos.y() > size().height() - 2) ||
         (pos.x() > 1 && pos.x() < size().width() - 2)) &&
        scene()->itemAt(mapToScene(pos)) != this) {
        return;
    }

    QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(layout());

    if (!lay) {
        return;
    }

    if (pos == QPoint()) {
        if (m_spacer) {
            lay->removeItem(m_spacer);
            m_spacer->hide();
        }
        return;
    }

    //lucky case: the spacer is already in the right position
    if (m_spacer && m_spacer->geometry().contains(pos)) {
        return;
    }

    Plasma::FormFactor f = formFactor();
    int insertIndex = -1;

    //FIXME: needed in two places, make it a function?
    for (int i = 0; i < lay->count(); ++i) {
        QRectF siblingGeometry = lay->itemAt(i)->geometry();

        if (f == Plasma::Horizontal) {
            qreal middle = (siblingGeometry.left() + siblingGeometry.right()) / 2.0;
            if (pos.x() < middle) {
                insertIndex = i;
                break;
            } else if (pos.x() <= siblingGeometry.right()) {
                insertIndex = i + 1;
                break;
            }
        } else { // Plasma::Vertical
            qreal middle = (siblingGeometry.top() + siblingGeometry.bottom()) / 2.0;
            if (pos.y() < middle) {
                insertIndex = i;
                break;
            } else if (pos.y() <= siblingGeometry.bottom()) {
                insertIndex = i + 1;
                break;
            }
        }
    }

    m_spacerIndex = insertIndex;
    if (insertIndex != -1) {
        if (!m_spacer) {
            m_spacer = new Spacer(this);
            m_spacer->panel = this;
        }
        lay->removeItem(m_spacer);
        m_spacer->show();
        lay->insertItem(insertIndex, m_spacer);
    }
}


}

#include "panel.moc"

