/*
 *   Copyright (C) 2008 Park "segfault" Joon-Kyu <mastermind@planetmono.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "pythonrunner.h"

#include <KIcon>

PythonRunner::PythonRunner( QObject* parent, const QVariantList &args )
    : Plasma::AbstractRunner(parent, args)
{
    Q_UNUSED(args)
    
    // initialize python
    Py_Initialize();
    globals_ = PyModule_GetDict(PyImport_AddModule("__main__"));

    setObjectName("Python Evaluator");
    setIgnoredTypes(Plasma::RunnerContext::Directory | Plasma::RunnerContext::File | 
                         Plasma::RunnerContext::NetworkLocation | Plasma::RunnerContext::Executable |
                         Plasma::RunnerContext::ShellCommand);
}

PythonRunner::~PythonRunner()
{
    if (PyErr_Occurred())
	PyErr_Clear();
    
    Py_Finalize();
}

void PythonRunner::match(Plasma::RunnerContext &context)
{
    QString cmd = context.query().trimmed();
    QString result;
    
    // detect prefix
    if (!cmd.startsWith("py="))
	return;
    
    cmd = cmd.remove(0, 3).trimmed();
    if (cmd.isEmpty())
	return;
    
    PyObject *call = PyRun_String(cmd.toLocal8Bit().data(), Py_eval_input, globals_, 0L);
    if (PyErr_Occurred()) {
	// FIXME: what should I do when error occurred?
	PyErr_Clear();
	
	return;
    }
    
    PyObject *repr = PyObject_Repr(call);
    result = QString::fromLocal8Bit(PyString_AsString(repr));
    Py_DECREF(call);
    Py_DECREF(repr);
        
    Plasma::QueryMatch match(this);
    match.setType(Plasma::QueryMatch::InformationalMatch);
    match.setIcon(KIcon("text-x-python"));
    match.setText(result);
    match.setData("py=" + result);
    match.setId(QString());
    context.addMatch(context.query(), match);
}

#include "pythonrunner.moc"
