/*
 *   Copyright (C) 2008 Petri Damsten <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "krossrunnerscript.h"
#include "searchcontextscript.h"
#include "searchmatchscript.h"
#include <Plasma/Package>
#include <plasma/abstractrunner.h>
#include <KDebug>

KrossRunnerScript::KrossRunnerScript(QObject* parent,
                                     const QVariantList &args,
                                     const QString &language)
    : Plasma::RunnerScript(parent), m_language(language)
{
    Q_UNUSED(args)
}

KrossRunnerScript::~KrossRunnerScript()
{
}

bool KrossRunnerScript::init()
{
    QString name = mainScript();
    if (runner() && !name.isEmpty()) {
        if (initScript(this, m_language, name, "runner")) {
            return true;
        }
    }
    kDebug() << "Could not init script" << m_language << name;
    return false;
}

void KrossRunnerScript::match(Plasma::RunnerContext &context)
{
    if (runner()) {
        SearchContextScript s(runner(), context);
        callFunction("match",
            QVariantList() << qVariantFromValue(qobject_cast<QObject*>(&s)));
    }
}

void KrossRunnerScript::run(const Plasma::RunnerContext &context,
                            const Plasma::QueryMatch &match)
{
    if (runner()) {
        // TODO: SearchContextScriptConst??
        SearchContextScript s(runner(), (Plasma::RunnerContext&)context);
        SearchMatchScript m(match);
        callFunction("run",
                QVariantList() <<
                qVariantFromValue(qobject_cast<QObject*>(&s)) <<
                qVariantFromValue(qobject_cast<QObject*>(&m)));
   }
}

QObject* KrossRunnerScript::newSearchMatch()
{
    SearchMatchScript *obj = new SearchMatchScript(runner());
    return qobject_cast<QObject*>(obj);
}

#include "krossrunnerscript.moc"
