
#include <KDebug>

#include "qedje_wallpaper.h"


K_EXPORT_PLASMA_WALLPAPER(qedje, QEdjeWallpaper)


QEdjeWallpaper::QEdjeWallpaper(QObject *parent, const QVariantList &args)
: Plasma::Wallpaper(parent, args), world(0), canvas(0),
    m_edje_group("e/desktop/background"), m_configWidget(0)
{
    Q_UNUSED(args);
}

QEdjeWallpaper::~QEdjeWallpaper()
{
    if (world) {
        world->hide();
        delete world;
        delete canvas;
    }
}

void QEdjeWallpaper::init(const KConfigGroup &config)
{
    canvas = new QZionCanvas();

    // setup edje file
    m_edje_file = config.readEntry("edje_file", "");

    // create qedje object
    world = new QEdje(canvas, m_edje_file, m_edje_group);

    // show qedje object and resize applet and plasmoid based on
    // the object's min size
    world->show();
    canvas->widget()->setGeometry(boundingRect().toRect());
    connect(canvas, SIGNAL(updated(const QRegion &)),
            this, SLOT(canvasUpdated(const QRegion &)));
}

QWidget* QEdjeWallpaper::createConfigurationInterface(QWidget* parent)
{
    m_configWidget = new QWidget(parent);
    m_ui.setupUi(m_configWidget);
    m_ui.edje_file->setText(m_edje_file);

    connect(m_configWidget, SIGNAL(destroyed(QObject*)), this, SLOT(configWidgetDestroyed(QObject*)));
    return m_configWidget;
}

void QEdjeWallpaper::configWidgetDestroyed(QObject*)
{
    m_configWidget = 0;
}

void QEdjeWallpaper::save(KConfigGroup &config)
{
    config.writeEntry("edje_file", m_edje_file);
}

void QEdjeWallpaper::paint(QPainter* painter, const QRectF& exposedRect)
{
    m_pixmap = QPixmap(boundingRect().size().toSize());
    canvas->widget()->render(&m_pixmap);
    painter->drawPixmap(exposedRect, m_pixmap, exposedRect);
}

void QEdjeWallpaper::canvasUpdated(const QRegion &r)
{
    QRectF exposedArea(r.boundingRect());
    emit update(exposedArea);
}

