/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "koratgroup.h"

#include <plasma/framesvg.h>
#include <plasma/widgets/frame.h>

#include <KDebug>

#include <QGraphicsLinearLayout>
#include <QGraphicsGridLayout>
#include <QSizePolicy>
#include <QApplication>
#include <QFontMetricsF>

namespace Korat
{
class GroupPrivate
{
public:
    GroupPrivate(Group *grp)
        : q(grp)
        , background(new Plasma::FrameSvg())
        , footer(new Plasma::Frame())
        , layout(new QGraphicsLinearLayout(Qt::Vertical))
        , upperLayout(new QGraphicsLinearLayout())
        , currentWeight(-1)
    {
    }

    void calculateWidths()
    {
        widths.clear();

//        kDebug() << widths;
    }

    void sizeUpdate()
    {
        calculateWidths();
        emit q->sizeChanged();
    }

    Group *q;

    Plasma::FrameSvg *background;
    Plasma::Frame *footer;

    QList<Item *> itemList;
    QList<int> weightList;

    QGraphicsLinearLayout *layout;
    QGraphicsLinearLayout *upperLayout;

    QList<qreal> widths;
    int currentWeight;
};

Group::Group(const QString &title,QGraphicsItem *parent)
    : Item(parent)
    , d(new GroupPrivate(this))
{
    //setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    setContentsMargins(0,0,0,0);

    d->background->setImagePath("widgets/background");
//    d->background->setElementPrefix("north");
    setTitle(title);

    d->upperLayout->setSpacing(0);
    d->upperLayout->setContentsMargins(0,0,0,0);
    //d->upperLayout->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Expanding);

    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);

    d->layout->addItem(d->upperLayout);
    d->layout->addItem(d->footer);
    setLayout(d->layout);

    themeUpdated();
}

Group::Group(const QIcon &icon,QGraphicsItem *parent)
    : Item(parent)
    , d(new GroupPrivate(this))
{
    setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    d->background->setImagePath("widgets/background");
//    d->background->setElementPrefix("north");
    setIcon(icon);

    d->upperLayout->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Expanding);

    d->layout->addItem(d->upperLayout);
    d->layout->addItem(d->footer);
    setLayout(d->layout);
    themeUpdated();
}

Group::~Group()
{
    delete d;
}

void Group::setEnabledBorders(Plasma::FrameSvg::EnabledBorders borders)
{
    d->background->setEnabledBorders(borders);
    themeUpdated();
}

QString Group::title() const
{
    return d->footer->text();
}

void Group::setTitle(const QString &title)
{
    d->footer->setText(title);
}

QIcon Group::icon() const
{
//    return d->footer->icon();
    return QIcon();
}

void Group::setIcon(const QIcon &icon)
{
//    d->footer->setIcon(icon);
}

void Group::addSeparator()
{

}

void Group::addItem(Item *item)
{
    d->itemList << item;
    connect(item,SIGNAL(sizeChanged()),this,SLOT(sizeUpdate()));
    d->upperLayout->addItem(item);
    d->calculateWidths();
}

QList<WeightList> Group::availableWidths()
{
    QList<WeightList> result;

    WeightList groupWeightList;
    //groupWeightList << (int)GroupNormal << (int)GroupIcon;
    groupWeightList << (int)GroupNormal;

    result << groupWeightList;

    foreach (Item *item, d->itemList) {
        result << item->availableWidths();
    }

    return result;
}

qreal Group::getWidth(const WeightList &weightList)
{
    qreal left,top,right,bottom;
    d->background->getMargins(left,top,right,bottom);

    if (weightList.at(0) == GroupIcon) {
        return SMALLEST_ICON_SIZE + left + right;
        //return SMALLEST_ICON_SIZE + left + right;
    }
    qreal width = 0;
    int i = 1;
    foreach (Item *item, d->itemList) {
        WeightList tempWeightList;
        tempWeightList = weightList.mid(i,item->availableWidths().size());

        width += item->getWidth(tempWeightList);
        i += tempWeightList.size();
    }

    //kDebug() << weightList << width + left + right;
    return width + left +right;
}

void Group::adjustWidth(const WeightList &weightList)
{
    qreal minWidth = getWidth(weightList);
    setMinimumWidth(minWidth);
    setMaximumWidth(minWidth);

    if (weightList.at(0) == GroupIcon) {
//        kDebug() << "Switch to icon mode.";
        return;
    }
    int i = 1;
    foreach (Item *item, d->itemList) {
        WeightList tempWeightList;
        tempWeightList = weightList.mid(i,item->availableWidths().size());

        item->adjustWidth(tempWeightList);
        i += tempWeightList.size();
    }

//    kDebug() << weightList << getWidth(weightList) << contentsRect();
}

void Group::resizeEvent(QGraphicsSceneResizeEvent * event)
{
    QGraphicsWidget::resizeEvent(event);
    d->background->resizeFrame(event->newSize());
    //kDebug() << event->newSize().width() << minimumWidth();
}

void Group::paint(QPainter *painter, 
        const QStyleOptionGraphicsItem *option, 
        QWidget *widget)
{
    Q_UNUSED(option);
    Q_UNUSED(widget);
    d->background->paintFrame(painter);
}

void Group::themeUpdated()
{
    qreal left,top,right,bottom;
    d->background->getMargins(left,top,right,bottom);
    QFontMetricsF fm(QApplication::font());
    
    qreal height = top + bottom + LARGEST_HEIGHT + d->footer->preferredHeight();
    setMinimumHeight(height);
    setMaximumHeight(height);
    setContentsMargins(left,top,right,bottom);
}

} // namespace Korat

#include "koratgroup.moc"
// vim: sw=4 sts=4 et tw=100
