/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "koratminibargroup.h"

#include <KDebug>

#include <QGraphicsLinearLayout>

namespace Korat
{
class MiniBarGroupPrivate
{
public:
    MiniBarGroupPrivate(MiniBarGroup *minibargroup)
        : q(minibargroup)
        , layout(new QGraphicsLinearLayout(Qt::Vertical))
        , layoutRowOne(new QGraphicsLinearLayout())
        , layoutRowTwo(new QGraphicsLinearLayout())
        , layoutRowThree(new QGraphicsLinearLayout())
        , largestSize(BoxMiniBarOneRow)
        , minimumSize(BoxMiniBarThreeRow)
    {

    }

    void sizeUpdate()
    {
        qreal width;

        minibarWidths.clear();
        widths.clear();
        widths << 0 << 0 << 0;

        foreach (MiniBar *minibar, minibarList) {
            QList<WeightList> minibarWeightList = minibar->availableWidths();
            WeightList weightList;
            weightList.clear();
            foreach (const WeightList &wl, minibarWeightList) {
                weightList << wl.last();
            }
            width = minibar->getWidth(weightList);
            minibarWidths << width;
            widths[0] += width;
        }
        widths[1] = widths[0];
        widths[2] = widths[0];
        qreal widthRowOne = 0;
        qreal widthRowTwo = 0;
        qreal widthRowThree = 0;
        int i,j,k;
        best1 = best2 = -1;
        for (i = 1; i < minibarList.size(); ++i) {
            widthRowOne = widthRowTwo = 0;
            for (j = 0; j < i; ++j) {
                widthRowOne += minibarWidths[j];
            }
            for (j = i; j < minibarList.size(); ++j) {
                widthRowTwo += minibarWidths[j];
            }
            if (widths[1] > qMax(widthRowOne, widthRowTwo)) {
                best1 = i;
                widths[1] = qMax(widthRowOne, widthRowTwo);
            }
        }
        best1 = best2 = -1;
        widthRowOne = widthRowTwo = widthRowThree = 0;
        for (i = 2; i < minibarList.size(); ++i) {
            widthRowOne = widthRowTwo = widthRowThree = 0;
            for (j = 1; j < i; ++j) {
                for (k = 0; k < j; ++k) {
                    widthRowOne += minibarWidths[k];
                }
                for (k = j; k < i; ++k) {
                    widthRowTwo += minibarWidths[k];
                }
                for (k = i; k < minibarList.size(); ++k) {
                    widthRowThree += minibarWidths[k];
                }
                if (widths[2] > qMax(qMax(widthRowOne, widthRowTwo), widthRowThree)) {
                    widths[2] = qMax(qMax(widthRowOne, widthRowTwo), widthRowThree);
                    best1 = j;
                    best2 = i;
                }
            }
        }
        //kDebug() << widths;
        emit q->sizeChanged();
    }

    MiniBarGroup *q;

    QList<MiniBar *> minibarList;

    QGraphicsLinearLayout *layout;
    QGraphicsLinearLayout *layoutRowOne;
    QGraphicsLinearLayout *layoutRowTwo;
    QGraphicsLinearLayout *layoutRowThree;

    QList<qreal> widths;
    QList<qreal> minibarWidths;

    int best1;
    int best2;

    LayoutWeight largestSize;
    LayoutWeight minimumSize;
};

MiniBarGroup::MiniBarGroup(QGraphicsItem *parent)
    : Item(parent)
    , d(new MiniBarGroupPrivate(this))
{
    setMinimumHeight(LARGEST_HEIGHT);
    setMaximumHeight(LARGEST_HEIGHT);
    d->layoutRowOne->setSpacing(0);
    d->layoutRowOne->setContentsMargins(0,0,0,0);
    d->layoutRowTwo->setSpacing(0);
    d->layoutRowTwo->setContentsMargins(0,0,0,0);
    d->layoutRowThree->setSpacing(0);
    d->layoutRowThree->setContentsMargins(0,0,0,0);

    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);

    d->layout->addItem(d->layoutRowOne);
    d->layout->addItem(d->layoutRowTwo);
    d->layout->addItem(d->layoutRowThree);

    setLayout(d->layout);
}

MiniBarGroup::~MiniBarGroup()
{

}

void MiniBarGroup::addMiniBar(MiniBar *minibar)
{
    d->minibarList << minibar;
    connect(minibar,SIGNAL(sizeChanged()),this,SLOT(sizeUpdate()));
//    d->layout->addItem(minibar);
    d->sizeUpdate();
}

void MiniBarGroup::removeMiniBar(int index)
{
    if ((index >= 0) && (index < d->minibarList.size())) {
        MiniBar *minibar = d->minibarList.takeAt(index);
        disconnect(minibar,SIGNAL(sizeChanged()),this,SLOT(sizeUpdate()));
        d->sizeUpdate();
    }
}

void MiniBarGroup::removeMiniBar(MiniBar *minibar)
{
    removeMiniBar(d->minibarList.indexOf(minibar));
}

MiniBar *MiniBarGroup::getMinibar(int index)
{
    return d->minibarList.at(index);
}

QList<WeightList> MiniBarGroup::availableWidths()
{
    QList<WeightList> result;
    WeightList weightList;

    weightList.clear();
    result.clear();

    if ((BoxMiniBarOneRow >= d->minimumSize) && (BoxMiniBarOneRow <= d->largestSize)) {
        weightList << (int)BoxMiniBarOneRow;
    }
    if ((BoxMiniBarTwoRow >= d->minimumSize) && (BoxMiniBarTwoRow <= d->largestSize)) {
        weightList << (int)BoxMiniBarTwoRow;
    }
    if ((BoxMiniBarThreeRow >= d->minimumSize) && (BoxMiniBarThreeRow <= d->largestSize)) {
        weightList << (int)BoxMiniBarThreeRow;
    }

    result << weightList;

    //kDebug() << result;
    return result;
}

qreal MiniBarGroup::getWidth(const WeightList &weightList)
{
    qreal width = 0;
    switch (weightList.at(0)) {
    case BoxMiniBarOneRow:
        width = d->widths[0];
        break;
    case BoxMiniBarTwoRow:
        width = d->widths[1];
        break;
    case BoxMiniBarThreeRow:
        width = d->widths[2];
        break;
    default:
        kWarning() << "Trouble:" << weightList;
    }
    //kDebug() << weightList << width;
    return width;
}

void MiniBarGroup::adjustWidth(const WeightList &weightList)
{
    qreal minWidth = getWidth(weightList);
    setMinimumWidth(minWidth);
    setMaximumWidth(minWidth);
    foreach (MiniBar *minibar, d->minibarList) {
        QList<WeightList> minibarWeightList = minibar->availableWidths();
        WeightList tempWeightList;
        tempWeightList.clear();
        foreach (const WeightList &wList, minibarWeightList) {
            // always pickup the smallest width ( last() ).
            tempWeightList << wList.last();
        }
        //kDebug() << tempWeightList;
        minibar->adjustWidth(tempWeightList);
    }
    while (d->layoutRowOne->count()) {
        d->layoutRowOne->removeAt(0);
    }
    while (d->layoutRowTwo->count()) {
        d->layoutRowTwo->removeAt(0);
    }
    while (d->layoutRowThree->count()) {
        d->layoutRowThree->removeAt(0);
    }
    switch (weightList.at(0)) {
    case BoxMiniBarOneRow:
        foreach (MiniBar *minibar, d->minibarList) {
            d->layoutRowTwo->addItem(minibar);
        }
        break;
    case BoxMiniBarTwoRow:
        if (d->best1 == -1) {
            foreach (MiniBar *minibar, d->minibarList) {
                d->layoutRowOne->addItem(minibar);
            }
        } else {
            for (int i = 0; i < d->best1; i++) {
                d->layoutRowOne->addItem(d->minibarList[i]);
            }
            for (int i = d->best1; i < d->minibarList.size(); ++i) {
                d->layoutRowTwo->addItem(d->minibarList[i]);
            }
        }
        break;
    case BoxMiniBarThreeRow:
        if ((d->best1 == -1) || (d->best2 == -1)) {
            foreach (MiniBar *minibar, d->minibarList) {
                d->layoutRowOne->addItem(minibar);
            }
        } else {
            for (int i = 0; i < d->best1; i++) {
                d->layoutRowOne->addItem(d->minibarList[i]);
            }
            for (int i = d->best1; i < d->best2; ++i) {
                d->layoutRowTwo->addItem(d->minibarList[i]);
            }
            for (int i = d->best2; i < d->minibarList.size(); ++i) {
                d->layoutRowThree->addItem(d->minibarList[i]);
            }
        }
        break;
    default:
        kWarning() << "Trouble:" << weightList;
    }
    //kDebug() << weightList << minWidth;
}

void MiniBarGroup::setLargestSizeMode(LayoutWeight mode)
{
    d->largestSize = mode;
}

void MiniBarGroup::setMinimumSizeMode(LayoutWeight mode)
{
    d->minimumSize = mode;
}

void MiniBarGroup::setFixedSizeMode(LayoutWeight mode)
{
    d->largestSize = d->minimumSize = mode;
}

} // namespace Korat

#include "koratminibargroup.moc"
// vim: sw=4 sts=4 et tw=100
