/*
    KUserFace - advanced user properties
    Copyright (C) 2005 Friedrich W. H. Kossebau <Friedrich.W.H@Kossebau.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301  USA
*/

#ifndef KUSERFACE_H
#define KUSERFACE_H

// qt specific
#include <qmime.h>
// kde specific
#include <kuser.h>
#include <kicontheme.h>
#include <ksharedptr.h>

class KUserGroup;
class KUser;
class QString;
class QImage;
class QPixmap;
class QIconSet;
class QStringList;

class KUserPrivate;
template <class T> class QValueList;
template <class T, class V> class QMap;


enum KUserDescStyle
{
  ShowFullName = 1,
  ShowLoginName = 2,
  OnlyFullName = 1,
  OnlyLoginName = 2,
  FirstFullName = 3,
  FirstLoginName = 7,
  NameMask = 7,
  TwoRows = 8,
  ShowFace = 16,
  ShowFaceBefore = 16,
  ShowFaceBehind = 48,
  ShowFaceMask = 48,
};


#ifndef KDECORE_EXPORT
#define KDECORE_EXPORT
#endif
/**
 * @short Creates rich text describing the user, e.g. for tooltips
 *
 * 
 *
 * @author Friedrich W. H. Kossebau <Friedrich.W.H@Kossebau.de>
 * @since 3.5
 */
class KDECORE_EXPORT KUserRichTexter
{
public:
  static KUserRichTexter *self();

protected:
  KUserRichTexter();
public:
  ~KUserRichTexter();

public:
  /**
   * @param User
   * @param Size 
   */
  QString createFaceURL( const KUser &User, int Size = 0 ) const;
  /**
   * @param User
   * @param Style
   * @param Complete true if complete richtext should be delivered with embracing <qt> tags
   */
  QString createUserDesc( const KUser &User, int Style = -1, bool Complete = true ) const;

private:
  class KPrivate;
  KPrivate * const d;
};


class KUserFaceLoaderPrivate;
/**
 * @short Loads the face of a user
 *
 * This class represents the face of a user on your system. You can fetch
 * pixmaps in different sizes. Instances of this class will be explicitly shared,
 * so copying objects is very cheap and you can safely pass objects by value.
 *
 * @author Friedrich W. H. Kossebau <Friedrich.W.H@Kossebau.de>
 * @since 3.5
 */
class KDECORE_EXPORT KUserFaceLoader {
  public:
    enum Policy { Public, Private };

  public:
    static KUserFaceLoader *self();

  protected:
  /**
   * Creates a new KUserFaceLoader.
   */
    KUserFaceLoader();

  public:
  /**
   * Deconstruktor
   */
    ~KUserFaceLoader();

  public:
  /**
   * @returns a pixmap of the face in the given size or a null pixmap if null
   */
  QImage loadFace(const KUser &user, int size = 0, KUserFaceLoader::Policy policy = KUserFaceLoader::Private);

  /**
   * @returns a pixmap of the face in the given size or a null pixmap if null
   */
  QPixmap loadFaceIcon(const KUser &user, KIcon::Group group,
                       int size = 0, int state = KIcon::DefaultState,
                       KUserFaceLoader::Policy policy = KUserFaceLoader::Private);

  /**
   * @returns a icon set of the face in the given size or a null set if null
   */
  QIconSet loadFaceIconSet(const KUser &user, KIcon::Group group,
                           int size = 0, KUserFaceLoader::Policy policy = KUserFaceLoader::Private);

  private:
    KUserFaceLoaderPrivate* const d;
};

/*
helper function as with KIconLoader. Not sure if we do want this.
Would be nice to have the same api style, but then this would only blow
the api while the code doesn't enhance much if writing
    IconSet = UserFaceSmallIconSet( User );
instead of
    IconSet = User.faceIconSet( KIcon::Small );
And we would have to write the functions for all 5 groups!

KDECORE_EXPORT QPixmap UserFaceSmallIcon(const KUser &user, int size=0,
                                    int state=KIcon::DefaultState);
KDECORE_EXPORT QIconSet UserFaceSmallIconSet(const KUser &user, int size=0);
*/


#endif
